! ============================================================	
! calculates derivatives
! ==========================================================

subroutine calc_derivatives(NN, n, nobs, H, w, wpop, XT99, distance, contig, lang, agree, uswage, Toptimal, XToptimal, xoptimal, Xderivatives) 
	use MSIMSL

	implicit none

	integer(4), intent(in) :: NN, n, nobs
	real(8), dimension(n), intent(in) :: xoptimal				! parameter guess
	real(8), dimension(NN), intent(in) :: H						! population	in millions
	real(8), dimension(NN), intent(in) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN), intent(in) :: wpop					! working population (as a faction of total)
	real(8), dimension(NN, NN), intent(in) :: XT99				! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN), intent(in) :: distance			! distance in km
	integer(4), dimension(NN, NN), intent(in) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN), intent(in) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN), intent(in) :: agree			! 1 for both countries in the same trade agreement--ASEAN, COMESA, EFTA, MERCOSUR, NAFTA, EU
	real(8), intent(in) :: uswage

	real(8), dimension(NN), intent(in) :: Toptimal				! T at xoptimal
	real(8), dimension(nobs), intent(in) :: XToptimal			! trade flows as predicted by the model
	real(8), dimension(nobs, n), intent(out) :: Xderivatives
	
	integer(4), parameter :: maxft = 20
	logical :: succt
	real(8):: obj
	real(8), dimension(n):: hjump_perc, xoptimal2, xplush
	real(8), dimension(nobs) :: XTxplush

	real(8) :: d1, d2, d3, b, l, teu, tagree, rd, denominator, numerator, hjump
	real(8), dimension(NN) :: T, tinit, gdp
	real(8), dimension(NN, NN) :: Dist, XT
	integer(4) :: j, jj, imp, exp, titer
	
	integer(4), dimension(NN, NN) :: aux
	real(8), parameter :: theta = 8.28


	! set optimal parameter values (instead of transformation of parameters used in optimization)

	xoptimal2 = xoptimal
	
	! size of derivative jump as a multiple of original parameter value
	hjump_perc = 1.01
	
! ============================================================================
! main loop : find derivative vector for each parameter guess
	
parameter_loop: do jj = 1, n
	
	xplush = xoptimal2
	xplush(jj) = xoptimal2(jj)*hjump_perc(jj)


! the lines below specify which parameters are log-normally distributed, and which are not.
	if ( (jj == 2) .or. (jj == 3) ) then
		hjump = xplush(jj) - xoptimal2(jj)
	else
		hjump = ( log(xplush(jj)) - log(xoptimal2(jj)) )
	end if 

	! assign new parameter values
	d1 = xplush(1)
	d2 = xplush(2)
	d3 = xplush(3)
	b = xplush(4)
	l = xplush(5)
	tagree = xplush(6)

	! construct iceberg cost matrix
	call iceberg(Dist, NN, distance, contig, lang, agree, d1, d2, d3, b, l, tagree)

	! =======================================================================
	! calculate excess demand vector
	T = Toptimal
	succt = .false.
	j = 0
	tinit = T
	do
		if ((succt) .or. (j>= maxft)) exit
		call random_number(rd)
		rd = rd/3
		T = rd*T + (1-rd)*tinit
		call findt(NN, H, wpop, w, Dist, theta, T, XT, succt, titer)
		j = j + 1
	end do
	j = 0
	do
		if ((succt) .or. (j>= maxft)) exit
		call findt(NN, H, wpop, w, Dist, theta, T, XT, succt, titer)
		j = j + 1
	end do
	
	if (.not. succt) then
		print *, "new technology vector not found"; stop
	end if

	do j = 1, NN
		XT(j,j) = 0
	end do

	XT = XT*uswage


! =========================================================================================
! trade flows in millions of US$ -- obj as suggested by Matt Wiswall

	gdp = uswage*w*wpop*H
	obj = 0.
	denominator = 0.
	j = 0
	do imp = 1, NN
		do exp = 1, NN
		if ( (imp/=exp) .and. (XT99(imp,exp) >= -.0000001)  ) then
			obj = obj + ((XT99(imp,exp) - XT(imp,exp))**2)/((gdp(imp)*gdp(exp))**2.)
			denominator = denominator + ((XT99(imp,exp))**2)/((gdp(imp)*gdp(exp))**2.)
			j = j + 1
			XTxplush(j) = XT(imp, exp)/(gdp(imp)*gdp(exp))
		end if
		end do
	end do
	obj = obj/denominator
	print *, "parameter, titer, obj", jj, titer, obj
	print *, " "

	Xderivatives(:, jj)	= (XToptimal - XTxplush)/hjump
	
	
end do parameter_loop

end subroutine calc_derivatives
