! ============================================================	
!calculates objective function
! ==========================================================

subroutine calc_obj(NN, n, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, succt, x, obj)
				   
	use MSIMSL

	implicit none

	integer(4), intent(in) :: NN, n
	real(8), dimension(n), intent(in) :: x						! parameter guess
	real(8), dimension(NN), intent(in) :: H						! population	in millions
	real(8), dimension(NN), intent(in) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN), intent(in) :: wpop					! working population (as a faction of total)
	real(8), dimension(NN, NN), intent(in) :: XT99				! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN), intent(in) :: distance			! distance in km
 	integer(4), dimension(NN, NN), intent(in) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN), intent(in) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN), intent(in) :: agree			! 1 for both countries in the same trade agreement
	real(8), intent(in) :: uswage

	real(8), dimension(NN), intent(inout) :: T
	logical, intent(out) :: succt
	real(8), intent(out) :: obj
	
	integer(4), parameter :: maxft = 15

	real(8) ::  d1, d2, d3, b, l, tagree, rd, residual
	real(8), dimension(NN) :: tinit, objperc
	real(8), dimension(NN, NN) :: Dist, XT
	integer(4) :: j, imp, exp

	! auxiliary variables for Matt Wiswall's objective function
	real(8), dimension(NN) :: gdp
	real(8) :: denominator, numerator, npos, nneg

	real(8), parameter :: theta = 8.28, &		! 8.28, 3.60 and 12.86 were EK's  estimates
						  weight = 2.0			! 0.703214070695504		! estimate of weights taken from a two-step estimation procedure     
	
	d1 = x(1)
	d2 = x(2)
	d3 = x(3)
	b = x(4)
	l = x(5)
	tagree = x(6)

	! =======================================================================
	! construct iceberg cost matrix

	call iceberg(Dist, NN, distance, contig, lang, agree, d1, d2, d3, b, l, tagree)


	! =======================================================================
	! estimate trade flows

	T = T/T(1)
	succt = .false.
	j = 0
	tinit = T
	do
		if ((succt) .or. (j>= maxft)) exit
		call random_number(rd)
		rd = rd/3
		T = rd*T + (1-rd)*tinit
		call findt(NN, H, wpop, w, Dist, theta, T, XT, succt)
		j = j + 1
	end do
	j = 0
	do
		if ((succt) .or. (j>= maxft)) exit
		call findt(NN, H, wpop, w, Dist, theta, T, XT, succt)
		j = j + 1
	end do
	
	do j = 1, NN
		XT(j,j) = 0.
	end do

	XT = XT * uswage

! =========================================================================================
! objective function

	gdp = uswage*w*wpop*H
	obj = 0.
	denominator = 0.
	j = 0
	do imp = 1, NN
		do exp = 1, NN
		if ( (imp/=exp) .and. (XT99(imp,exp) >= -.0000001)  ) then
			obj = obj + ((XT99(imp,exp) - XT(imp,exp))**2)/((gdp(imp)*gdp(exp))**weight)
			denominator = denominator + ((XT99(imp,exp))**2)/((gdp(imp)*gdp(exp))**weight)
			j = j + 1
		end if
		end do
	end do
	obj = obj/denominator

end subroutine calc_obj
