! ============================================================	
!calculates objective function
! ==========================================================

subroutine calc_optimal(NN, n, nobs, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, succt, x, obj, XTdata, XToptimal, residuals, obsimpexp)

	use MSIMSL

	implicit none

	integer(4), intent(in) :: NN, n, nobs
	real(8), dimension(n), intent(in) :: x						! parameter guess
	real(8), dimension(NN), intent(in) :: H						! population	in millions
	real(8), dimension(NN), intent(in) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN), intent(in) :: wpop					! working population (as a faction of total)
	real(8), dimension(NN, NN), intent(in) :: XT99				! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN), intent(in) :: distance			! distance in km
 	integer(4), dimension(NN, NN), intent(in) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN), intent(in) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN), intent(in) :: agree			! 1 for both countries in the same trade agreement
	real(8), intent(in) :: uswage

	real(8), dimension(NN), intent(inout) :: T
	real(8), dimension(nobs), intent(out) :: XTdata, XToptimal, residuals
	logical, intent(out) :: succt
	real(8), intent(out) :: obj
	integer(4), dimension(nobs, 2), intent(out) :: obsimpexp
	
	integer(4), parameter :: maxft = 15

	real(8) ::  d1, d2, d3, b, l, tagree, rd, residual
	real(8), dimension(NN) :: tinit
	real(8), dimension(NN, NN) :: Dist, XT
	integer(4) :: j, imp, exp, titer

	! auxiliary variables for Matt Wiswall's objective function
	real(8), dimension(NN) :: gdp
	real(8) :: denominator, numerator, npos, nneg

	real(8), parameter :: theta = 8.28
	
	d1 = x(1)
	d2 = x(2)
	d3 = x(3)
	b = x(4)
	l = x(5)
	tagree = x(6)

	! =======================================================================
	! construct iceberg cost matrix

	call iceberg(Dist, NN, distance, contig, lang, agree, d1, d2, d3, b, l, tagree)

	! =======================================================================
	! calculate excess demand vector

	succt = .false.
	j = 0
	call findt(NN, H, wpop, w, Dist, theta, T, XT, succt, titer)
	
	do j = 1, NN
		XT(j,j) = 0.
	end do

	XT = XT * uswage

! =========================================================================================
! trade flows in millions of US$ -- obj as suggested by Matt Wiswall

	gdp = uswage*w*wpop*H
	obj = 0.
	denominator = 0.
	j = 0
	do imp = 1, NN
		do exp = 1, NN
		if ( (imp/=exp) .and. (XT99(imp,exp) >= -.0000001)  ) then
			obj = obj + ((XT99(imp,exp) - XT(imp,exp))**2)/((gdp(imp)*gdp(exp))**2)
			denominator = denominator + ((XT99(imp,exp))**2)/((gdp(imp)*gdp(exp))**2)
			j = j + 1
			XTdata(j) = XT99(imp, exp)/(gdp(imp)*gdp(exp))
			XToptimal(j) = XT(imp, exp)/(gdp(imp)*gdp(exp))
			residuals(j) = XTdata(j) - XToptimal(j)
			obsimpexp(j,:) = (/ imp, exp /)
		end if
		end do
	end do
	obj = obj/denominator
	print *, " ----------------------------------- "
	print *, "tests below must equal zero"
	print *, "number of observations: ", j - nobs
	print *, "objective function:              ", obj - sum(residuals**2)/denominator
	print *, "objective function = ", obj
	print *, " ----------------------------------- "
	print *, " "


end subroutine calc_optimal

