!     excdem.F90      calculates excess demand vector, and expenditure and labor demand matrix
 
subroutine excdemand(NN, H, wpop, w, Dist, T, lambda, XT, XA, XB, LT, z, zp, alpha, theta, thetab, sigmaa, sigmab, gammata, gammatb)

	use MSIMSL
	implicit none
	
	integer(4), intent(in) :: NN
	real(8), intent(in) :: theta, alpha, gammata, gammatb, sigmaa, sigmab, thetab
	real(8), dimension(NN, NN), intent(in) :: Dist
	real(8), dimension(NN), intent(in) :: H, wpop, w, T
	real(8), dimension(NN), intent(inout) :: lambda

	real(8), dimension(NN), intent(out) :: z, zp
	real(8), dimension(NN, NN), intent(out) :: XT, LT, XA, XB

	integer(4):: imp, exp
	real(8), dimension(NN, NN):: XX
	real(8), dimension(NN):: num

	! ==============================================
	! per capita expenditure matrix


	call ilambdas(NN, H, wpop, w, Dist, T, lambda, XX, XA, XB, alpha, theta, thetab, sigmaa, sigmab, gammata, gammatb)

	
! =============================================
! total expenditures and labor demand matrices

	do exp = 1, NN
		do imp = 1, NN
			XT(imp, exp) = XX(imp, exp)*H(imp)
			LT(imp, exp) = XT(imp, exp)/w(exp)
		end do
	end do
	
	! ============================================
	! excess demand vectors (total and as a percentage of population)

	z = sum(LT, DIM = 1) - wpop * H
	zp = z/(wpop*H)

end subroutine excdemand

! ==========================================================================
! ==========================================================================

subroutine ilambdas(NN, H, wpop, w, Dist, T, lambda, XX, XA, XB, alpha, theta, thetab, sigmaa, sigmab, gammata, gammatb)
	use MSIMSL
	implicit none
	
	real(8), parameter :: toll = 5e-8
	integer(4), parameter :: maxi = 3000

	integer(4), intent(in) :: NN
	real(8), intent(in) :: theta, sigmab, gammatb, sigmaa, alpha, gammata, thetab
	real(8), dimension(NN, NN), intent(in) :: Dist
	real(8), dimension(NN), intent(in) :: H, wpop, w, T
	real(8), dimension(NN), intent(inout):: lambda
	real(8), dimension(NN, NN), intent(out):: XX, XA, XB
	
	integer(4):: imp, exp, counti
	real(8), dimension(NN):: ew, ximp, num, numb, xhimp, qshare, qnum, qshare2
	real(8) :: error, errold, spsi, rd, ctrlam, lamold, hshare, nhlam, auxa, auxb

	do imp = 1, NN
		ew = (Dist(imp, :))*w
		spsi = 0.5
		counti = 1
		ctrlam = lambda(imp)

		do exp = 1, NN
			num(exp) = T(exp)*(Dist(imp, exp)*w(exp))**(-theta) 
			numb(exp)= T(exp)*(Dist(imp, exp)*w(exp))**(-thetab) 
		end do

		auxa = (alpha**(sigmaa))*((sum(num))**((sigmaa - 1)/theta))*gammata
		auxb = ((1 - alpha)**(sigmab))*((sum(numb))**((sigmab - 1)/thetab))*gammatb
		
		! initial guess of lambda (if expenditures in A are greater than wealth)
		if ( ctrlam <= (auxa/(wpop(imp)*w(imp)))**(1/sigmaa) ) then
			if (alpha == 1) then
				ctrlam = (auxa/(wpop(imp)*w(imp)))**(1/sigmaa)
			else
				ctrlam = ((auxa/(wpop(imp)*w(imp)))**(1/sigmaa))*1.1
			end if
		end if
			

		! calculate expenditures for a given lambda
		! ================================================================
		! sector A
		hshare = (ctrlam**(-sigmaa))*auxa
		xhimp = (num/sum(num))*hshare
		
		! sector B
		hshare = (ctrlam**(-sigmab))*auxb
		ximp = (numb/sum(numb))*hshare
		
		! total
		ximp = ximp + xhimp
		! ==================================================================

		error = sum(ximp) - wpop(imp)*w(imp)
		lamold = ctrlam
		errold = error

		implamb: do
			! stop controls
			!==========================================================
			if (abs(error) < toll) then
				lambda(imp) = ctrlam
				exit implamb
			else if ( counti >= maxi) then
				print *, "lambdas did not converge"
				stop
				exit implamb
			end if

			! ============================================================
			! new guess
			! ============================================================
			ctrlam = lamold*(1. + spsi*error)
			if (ctrlam < 0) then
				spsi = spsi*(1.e-1)
				ctrlam = 0.9*lamold
			end if

			! ============================================================
			! calculate expenditures for a given lambda
			! ================================================================
			! sector A
			hshare = (ctrlam**(-sigmaa))*auxa
			xhimp = (num/sum(num))*hshare

			! sector B
			hshare = (ctrlam**(-sigmab))*auxb
			ximp = (numb/sum(numb))*hshare

			! total
			ximp = ximp + xhimp
			! ==================================================================
			
			error = sum(ximp) - wpop(imp)*w(imp)			! uses per capita income to clear the budget constraint

			call random_number(rd)
			if (rd > (49./50.)) then
				spsi = max(min(10., spsi), 0.01)
			else if (abs(error + errold) < abs(error) + abs(errold)) then
			! decrease sensitivity if error changed sign
				spsi = spsi*(0.9 + rd/8.)
			! increase otherwise
			else 
				spsi = spsi*(1.2 + rd/8.)
			end if		

			if (abs(error) > abs(errold)) then
				error = errold
				ctrlam = lamold
			else
				lamold = ctrlam
				errold = error
			end if
			
			counti = counti + 1

		end do implamb
		XX(imp, :) = ximp
		XA(imp, :) = xhimp
		XB(imp, :) = (ximp - xhimp)

!		qshare(imp) = sum(xhimp)/sum(ximp)
!		xhimp(imp) = 0
!		ximp(imp) = 0
!		qshare2(imp) = sum(xhimp)/sum(ximp)
	
	end	do

!	open(1, FILE='qualitygds_share.txt', STATUS='REPLACE')
!		write (1,*), "share of sector A goods"
!		do exp = 1, NN
!			write(1,*), qshare(exp)
!		end do
!		write (1,*), "  "
!		write (1,*), "share imports from sector A"
!		do exp = 1, NN
!			write(1,*), qshare2(exp)
!		end do
!	close(1)
	

end subroutine ilambdas
