!     cluster_paramvar.F90	estimates the variance-covariance matrix for parameters with clustering by exporter
! =========================================

subroutine cluster_paramvar(NN, n, nobs, Xderivatives, residuals, obsimpexp, varparam)
						   
	use MSIMSL
	implicit none

	integer(4), intent(in) :: NN, n, nobs
	real(8), dimension(nobs, n), intent(in)	:: Xderivatives
	real(8), dimension(nobs), intent(in) :: residuals
	integer, dimension(nobs, 2), intent(in) :: obsimpexp
	real(8), dimension(n, n), intent(out) :: varparam

	integer(4) :: impo, expo, sizecluster, j, jj
	integer(4), dimension(nobs) :: iscluster
	real(8), dimension(n,n) :: xcxc, xcuuxc, auxm1				! auxiliary matrices
	real(8), dimension(n) :: auxv
	real(8), dimension(:, :), allocatable ::  xc
	real(8), dimension(:), allocatable :: uc

	! =========================================================
	!  data clustered by exporter
	
	xcxc = 0.
	xcuuxc = 0.
	varparam = 0.

	expo_cluster: do expo = 1, NN
		! count the number of observations with importer equal to expo
		iscluster = 0
		where(obsimpexp(:,2) == expo) iscluster = 1
		sizecluster = sum(iscluster)
		
		! determine size of xc, pseudo-regressors meatrix, and uc, residuals matrix
		allocate(xc(sizecluster, n))
		allocate(uc(sizecluster))

		! construct xc and uc
		jj = 0
		do j = 1, nobs
			if (iscluster(j)==1) then
				jj = jj + 1
				xc(jj,:) = xderivatives(j,:)
				uc(jj) = residuals(j)
			end if
		end do
		
		! calculate X_c'X_c
		call DMXTXF(sizecluster, n, xc, sizecluster, n, auxm1, n)
		xcxc = xcxc + auxm1
		
		! calculate X_c'u_c u_c'X_c
		j = 2
		call DMURRV(sizecluster, n, xc, sizecluster, sizecluster, uc, j, n, auxv)
		deallocate(uc); deallocate(xc)

		do j = 1, n
			do jj = 1, n
				auxm1(j,jj) = auxv(j)*auxv(jj)
			end do
		end do
		xcuuxc = xcuuxc + auxm1

	end do expo_cluster
	
	! invert xcxc
	call DLINRG(n, xcxc, n, xcxc, n)
				
	! get var parameter
	call DMRRRR(n, n, xcxc, n, n, n, xcuuxc, n, n, n, auxm1, n)
	call DMRRRR(n, n, auxm1, n, n, n, xcxc, n, n, n, varparam, n)
	
	
	end subroutine cluster_paramvar
