! uses the delta method to calculate the variance-covariance matrix of the parameters of the model
! given the var-cov matgrix of their transformation
! ==========================================================

subroutine deltavar(n, x, varparam)

	use MSIMSL

	implicit none

	integer(4), intent(in) :: n
	real(8), dimension(n), intent(in) :: x			
	real(8), dimension(n,n), intent(inout) :: varparam
	
	! internal variables
	real(8), dimension(n, n) :: Gmatrix, auxm
	real(8) :: d1, d2, d3, b, l, tagree, alpha, thetab, sigmab
						  		

	d1 = x(1)
	d2 = x(2)
	d3 = x(3)
	b = x(4)
	l = x(5)
	tagree = x(6)
	alpha = 1./(x(7) + 1)		! homogeneous goods share
	thetab = 1 + x(8)			! theta for sector B
	sigmab = (x(9)/(x(9)+1))*thetab + 1.	!sigmaa must be between (theta + 1) and 1
	
	! Gmatrix is the matrix of derivatives evaluated at the parameter values
	Gmatrix = 0.d0	
	Gmatrix(1,1) = d1
	Gmatrix(2,2) = 1.d0
	Gmatrix(3,3) = 1.d0
	Gmatrix(4,4) = b
	Gmatrix(5,5) = l
	Gmatrix(6,6) = tagree
	Gmatrix(7,7) = alpha*(1.-alpha)
	Gmatrix(8,8) = thetab - 1.d0
	Gmatrix(9,9) = sigmab - 1.d0

	call DMRRRR(n, n, Gmatrix, n, n, n, varparam, n, n, n, auxm, n)
	call DMXYTF(n, n, auxm, n, n, n, Gmatrix, n, n, n, varparam, n)

end subroutine deltavar

