! uses the delta method to calculate the variance-covariance matrix of the parameters of the model
! given the var-cov matgrix of their transformation
! ==========================================================

subroutine deltavar(NN, n, x, varparam)

	use MSIMSL

	implicit none

	integer(4), intent(in) :: NN, n
	real(8), dimension(n), intent(in) :: x			
	real(8), dimension(n,n), intent(inout) :: varparam
	
	! internal variables
	real(8), dimension(n, n) :: Gmatrix, auxm
	real(8), dimension(NN) :: xshareA
	real(8) :: d1, d2, d3, b, l, tagree, thetab
	integer(4) :: j, jj
						  		

	d1 = x(1)
	d2 = x(2)
	d3 = x(3)
	b = x(4)
	l = x(5)
	tagree = x(6)
	thetab = 1 + x(7)			! theta for sector B
	xshareA = x(n-NN+1:n)/(1+x(n-NN+1:n))

		
	! Gmatrix is the matrix of derivatives evaluated at the parameter values
	Gmatrix = 0.d0	
	Gmatrix(1,1) = d1
	Gmatrix(2,2) = 1.d0
	Gmatrix(3,3) = 1.d0
	Gmatrix(4,4) = b
	Gmatrix(5,5) = l
	Gmatrix(6,6) = tagree
	Gmatrix(7,7) = thetab - 1.d0
	jj = 0
	do j = n-NN+1,n
	jj = jj + 1
	Gmatrix(j,j) = xshareA(jj)*(1. - xshareA(jj))
	end do

	if (jj/=NN) print *, "there is an error in the program deltavar"

	call DMRRRR(n, n, Gmatrix, n, n, n, varparam, n, n, n, auxm, n)
	call DMXYTF(n, n, auxm, n, n, n, Gmatrix, n, n, n, varparam, n)

end subroutine deltavar

