!     calc_obj.F90      minimizes a function using the polytope method

! ============================================================	
!	generates simulated data
! ==========================================================

subroutine gen_data(NN, n, H, w, wpop, XTdata, distance, contig, lang, agree, uswage, T, lambda, succt, x, quadrante)
	use MSIMSL

	implicit none

	integer(4), intent(in) :: NN, n
	real(8), dimension(n), intent(in) :: x						! parameter guess
	real(8), dimension(NN), intent(in) :: H						! population	in millions
	real(8), dimension(NN), intent(in) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN), intent(in) :: wpop					! working population (as a faction of total)
	real(8), dimension(NN, NN), intent(in) :: distance			! distance in km
	integer(4), dimension(NN, NN), intent(in) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN), intent(in) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN), intent(in) :: agree				! 1 for both countries in the EU
	real(8), intent(in) :: uswage

	real(8), dimension(NN), intent(inout) :: T, lambda
	logical, intent(out) :: succt
	integer(4), intent(out) :: quadrante
	real(8), dimension(NN, NN), intent(out) :: XTdata			
	
	integer(4), parameter :: maxft = 20

	real(8) :: d1, d2, d3, b, l, tagree, rd, gammatb, gammata, thetab , alpha, sigmab, denominator, numerator, npos, nneg
	real(8), dimension(NN) :: tinit, gdp, objperc			!objperc is the objective function per country
	real(8), dimension(NN, NN) :: Dist, XT99mil
	integer(4) :: j, imp, exp

	real(8), parameter :: theta = 8.28, &				!8.28, 3.60 and 12.86 are the estimates in EK
						  sigmaa = 5.00					! Broda and Weinstein

	d1 = x(1)
	d2 = x(2)
	d3 = x(3)
	b = x(4)
	l = x(5)
	tagree = x(6)
	alpha = 1./(x(7) + 1)		! homogeneous goods share
	thetab = 1 + x(8)			! theta for sector B
	sigmab = (x(9)/(x(9)+1))*thetab + 1.	!sigmaa must be between (theta + 1) and 1

	gammata = DGAMMA((theta + 1. - sigmaa)/theta)	! sector A
	gammatb = DGAMMA((thetab + 1. - sigmab)/thetab)	! sector B

	if (thetab > theta) then
		if (sigmab > sigmaa) then
			quadrante = 1
		else
			quadrante = 2
		end if
	else
		if (sigmab < sigmaa) then
			quadrante = 3
		else
			quadrante = 4
		end if
	end if
!	print *, "simulation in quadrant", quadrante

!print *, "alpha", alpha
!print *, "sigmaa", sigmaa
!print *, "sigmab", sigmab
!print *, "thetaa", theta
!print *, "thetab", thetab
!print *, "gammata", gammata
!print *, "gammatb", gammatb
!print *, " =============================== "
!print *, " "

	! =======================================================================
	! construct iceberg cost matrix

	call iceberg(Dist, NN, distance, contig, lang, agree, d1, d2, d3, b, l, tagree)

	! =======================================================================
	! calculate excess demand vector

	succt = .false.
	j = 0
	tinit = T
	do
		if ((succt) .or. (j>= maxft)) exit
		call random_number(rd)
		rd = rd/3
		T = rd*T + (1-rd)*tinit
		call findt(NN, H, wpop, w, Dist, T, lambda, XTdata, alpha, succt, theta, thetab, sigmaa, sigmab, gammata, gammatb)
		j = j + 1
	end do
	j = 0
	do
		if ((succt) .or. (j>= maxft)) exit
		call findt(NN, H, wpop, w, Dist, T, lambda, XTdata, alpha, succt, theta, thetab, sigmaa, sigmab, gammata, gammatb)
		if (j == 1) then
			T = w**(1/theta)
		else if (j == 2) then
			T = w**(1/thetab)
		else
			T = w**(1/(alpha*theta + (1 - alpha)*thetab) )
		end if
		j = j + 1
	end do
	
	do j = 1, NN
		XTdata(j,j) = 0
	end do

	XTdata = XTdata*uswage

!	open(2, file = 'teste.csv', STATUS ='REPLACE')
!	do exp = 1, NN
!		do imp = 1, NN
!			write(2,*), XTdata(imp, exp)
!		end do
!	end do

end subroutine gen_data
