!     iceberg.F90      calculates matrix of trade costs

subroutine iceberg(Dist, NN, distance, contig, lang, agree, d1, d2, d3, b, l, tagree)

	use MSIMSL
	implicit none
	
	integer(4), intent(in) :: NN
	real(8), intent(in) :: d1, d2, d3, b, l, tagree
	real(8), dimension(NN, NN), intent(in) :: distance		! distance in km
	integer(4), dimension(NN, NN), intent(in) :: contig		! 1 for contiguity
	integer(4), dimension(NN, NN), intent(in) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN), intent(in) :: agree			! 1 for both countries of the same trade agreement (see list in readata_imf)

	real(8), dimension(NN, NN), intent(out) :: Dist

	integer(4) :: exp, imp

	! basic distance matrix
	! ========================================================================

	Dist = d1 + d2 * distance + d3 * (distance**2)
	! flatten at top so that costs are a non-decreasing function of distance:
	where (distance >= - d2/(2*d3)) Dist = d1 - (d2**2)/(4*d3)

	where (Dist<1.) Dist = 1.
	
	! =========================================================================
	! PARAMETERS THAT MULTIPLY TRADE COSTS: BORDER, LANGUAGE, ETC
	! =========================================================================

	! contiguity
	! ========================================================================

	do imp = 1, NN
		do exp = 1, NN
			if (contig(imp, exp) ) Dist(imp, exp) = (Dist(imp, exp) - 1)*b + 1
		end do
	end do

	! language
	! ========================================================================

	do imp = 1, NN
		do exp = 1, NN
			if (lang(imp, exp) == 1) Dist(imp, exp) = (Dist(imp, exp) - 1)*l + 1
		end do
	end do

	! Trade agreement
	! ========================================================================

	do imp = 1, NN
		do exp = 1, NN
			if (agree(imp, exp) == 1) Dist(imp, exp) = (Dist(imp, exp) - 1)*tagree + 1
		end do
	end do

	! ==========================================================================
	! substitute main diagonal to 1
	! ==========================================================================
	
	do exp = 1, NN
		Dist(exp, exp) = 1
	end do


end subroutine iceberg
