!****************************************************************************
!
!  PROGRAM: Estimates the demand parameters in the 2nd stage of the NLLS estimation with inequalities within countries
!
!****************************************************************************

program main
	use MSIMSL

	implicit none
	
	! data variables
	! ==========================================================	
	integer(4), parameter :: NN = 162				!number of countries
	real(8), dimension(NN) :: w						!wages 1999 (normalized to US = 1)
	integer(4), dimension(NN) :: availability
	real(8), dimension(NN) :: w_highest, w_fourth, w_third, w_second, w_lowest

	! model variables/parameters
	! ===============================================================
	real(8), dimension(NN) :: myphia, myphib, xsharea, mylambda				!auxiliary variables to get prices, expenditure share in type A goods
	real(8) :: thetaa, thetab

	! polytope variables
	! ===============================================================	
	integer(4), parameter :: n = 2					!number of parameters to estimate
	real(8), dimension(n), parameter ::  xup = (/8., 4./), &
											xlow = (/ 1.1, .5/)

	integer(4), parameter :: np1 = n + 1	
	integer(4), parameter :: two = 2
	real(8), parameter :: usual_delta = 0.07, zero_term_delta = 0.00025, &
							prho = 1., pchi = 2., ppsi = 0.5, psigma = 0.5, &
							tol_fun = 5e-4, tol_par = 1e-3, &		!tolerance within polytope
							ptol_fun = 5e-4, ptol_par = 1e-3		!tolerance comparing polytopes
	integer(4), parameter ::  maxfun = 200*n, maxiter = 200*n		! maxfun = 10, maxiter = 10 ! 
	integer(4), parameter:: maxj = 5, max_xiguess = 10				!runs at most 5 polytopes per initial guess
	real(8), dimension(n, np1) :: v
	real(8), dimension(np1) :: fv
	real(8), dimension(n) :: xinitial
	integer :: startime, endtime, hz
	integer(4) :: j, xiguess

	real(8) ::  obj
	
	! ================================================================
	CALL SYSTEM_CLOCK(COUNT_RATE=hz)
	CALL SYSTEM_CLOCK(COUNT=startime)

	! ======================================================================
	! read data

	open(16, file = 'myphia.dat', STATUS ='OLD')
	open(17, file = 'myphib.dat', STATUS ='OLD')
	open(18, file = 'xsharea.dat', STATUS ='OLD')

	do j = 1, NN
		read(16,*), myphia(j)
		read(17,*), myphib(j)
		read(18,*), xsharea(j)
	end do
	close(16); close(17); close(18)

	call readata(NN, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest)


	! =======================================================================
	! read initial guesses
	
	thetaa = 8.28
	thetab = 1. + 18.2714879758774						! estimate from first stage
	xinitial(1) = 0.468162604811896    					! alpha 
	xinitial(2) = 0.146579190731382						! sigmaB
 	
	open(2, file = 'init_lambdaun.txt', STATUS ='OLD')
		do j = 1, NN
			read(2,*), mylambda(j)
		end do
	close(2)

! ===================================================================================
! call objective function with initial guess, and display result (not needed)

	call calc_obj(NN, n, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda, xinitial, obj)
	print *, "xinitial", xinitial(1:n)
	print *, "obj     ", obj

! ONE POLYTOPE CYCLE IS RUN
! ======================================================================================
	call polytope(n, np1, two, usual_delta, zero_term_delta, prho, pchi, ppsi, psigma, tol_fun, &
					tol_par, ptol_fun, ptol_par, maxfun, maxiter, maxj, xinitial, v, fv, &
					NN, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda)

	! ==========================================================================
	! save results

	1 Format(f13.9) 

	open(1, FILE='result_2stg.txt', STATUS='REPLACE')
		write(1,*), fv(1)
		write(1,*), ' '

		do j = 1, n
			write(1,*), v(j,1)
		end do
		write(1,*), ' '
		write(1,*), "lambdas"
		do j = 1, NN
			write(1,*), mylambda(j)
		end do
	close(1)

	print *, "obj    ", fv(1)
	print *, "alpha  ", 1./(v(1,1) + 1)
	print *, "sigmaB", (v(2,1)/(v(2,1)+1))*thetab + 1.
	print *, " "
	print *, " ---------------------------------------- "

! ======================================================================================
! SEVERAL (MAXIGUESS) POLYTOPE CYCLES ARE RUN
! ======================================================================================

xiguess = 1

do
	if (xiguess >= max_xiguess) exit

	! construct new guess
	! =========================================================
	do j = 1, n
		call random_number(xinitial(j))
	end do
	xinitial = (xinitial * (xup - xlow)) + xlow
	
	call calc_obj(NN, n, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda, xinitial, obj)
	print *, "xinitial", xinitial
	print *, obj

	call polytope(n, np1, two, usual_delta, zero_term_delta, prho, pchi, ppsi, psigma, tol_fun, &
					tol_par, ptol_fun, ptol_par, maxfun, maxiter, maxj, xinitial, v, fv, &
					NN, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda)

	! save 
	! =========================================================
	
	select case(xiguess)

		case(1)
			open(1, FILE='result_2stg1.txt', STATUS='REPLACE')
		case(2)
			open(1, FILE='result_2stg2.txt', STATUS='REPLACE')
		case(3)
			open(1, FILE='result_2stg3.txt', STATUS='REPLACE')
		case(4)
			open(1, FILE='result_2stg4.txt', STATUS='REPLACE')
		case(5)
			open(1, FILE='result_2stg5.txt', STATUS='REPLACE')
		case(6)
			open(1, FILE='result_2stg6.txt', STATUS='REPLACE')
		case(7)
			open(1, FILE='result_2stg7.txt', STATUS='REPLACE')
		case(8)
			open(1, FILE='result_2stg8.txt', STATUS='REPLACE')
		case(9)
			open(1, FILE='result_2stg9.txt', STATUS='REPLACE')
		case(10)
			open(1, FILE='result_2stg10.txt', STATUS='REPLACE')

	end select
	
		write(1,*), fv(1)
		write(1,*), ' '
	
		do j = 1, n
			write(1,*), v(j,1)
		end do
		write(1,*), "lambdas"
		do j = 1, NN
			write(1,*), mylambda(j)
		end do

	close(1)
	! ================================================================
	
	print *, "obj    ", fv(1)
	print *, "alpha  ", 1./(v(1,1) + 1)
	print *, "sigmaB", (v(2,1)/(v(2,1)+1))*thetab + 1.
	print *, " "
	print *, " ---------------------------------------- "


	xiguess = xiguess + 1

end do	

	CALL SYSTEM_CLOCK(COUNT=endtime)
	print*, " "
	print*, "=========================================================================="
	print*, "Time: ", DBLE(endtime - startime)/(60*DBLE(hz)), " minutes"
	print*, " "
	print*, " "	

end program main

