
!****************************************************************************

program main
	use MSIMSL 

	implicit none
	
	! data variables
	! ==========================================================	
	integer(4), parameter :: NN = 162				!number of countries
	real(8), dimension(NN) :: realw99				!wages 1999 (normalized to US = 1)	
	real(8), dimension(NN) :: H						!population in millions
	real(8), dimension(NN) :: wpop					!working population (as a faction of total)
	real(8), dimension(NN, NN) :: XT99				!trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN) :: distance			!distance in km
	integer(4), dimension(NN, NN) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN) :: agree			! 1 for both countries in the EU
	real(8) :: uswage

	! model variables/parameters
	! ================================================================
	integer(4), parameter :: n = 9			!number of parameters
	real(8), dimension(n) :: x
	real(8), dimension(NN) :: T0, T1, lambda, tradeshare0, tradeshare1
	real(8), dimension(NN) :: w, realw0, realw1					!wages 1999 (normalized to US = 1)	
	logical :: succt
	real(8) :: obj
	
	!parameters
	real(8) :: d1, d2, d3, b, l, tagree, rd, gammatb, gammata, thetab , alpha, sigmab 
	real(8), parameter :: theta = 8.28, &	!8.28, 3.60 and 12.86 are the estimates in EK
						  sigmaa = 5.0		! Broda and Weinstein mean estimate for SITC3

	integer(4), parameter :: maxft = 15, maxloop = NN
	real(8), dimension(NN) :: UA, UB, UT0, UT1, PA0, PB0, PA1, PB1, PAB0, PAB1
	real(8), dimension(NN, NN) :: Dist, XT, XA, XB

	! auxiliary variables
	! ================================================================
	integer :: startime, endtime, hz
	integer(4) :: j, imp, exp, countt, shock
	real(8), dimension(NN) :: XAn, XBn, XAi, XBi, &
							  XATn0, XATn1, &		! share of sector A in expenditures -- before and after change 
							  XATi0, XATi1, &		! share of sector A in production   -- before and after change 
							  aux1, aux2, winit		! auxiliary variables


	! ================================================================
	CALL SYSTEM_CLOCK(COUNT_RATE=hz)
	CALL SYSTEM_CLOCK(COUNT=startime)

	! ======================================================================
	! read data
	! =======================================================================

	call readata(NN, H, realw99, wpop, XT99, distance, contig, lang, agree, uswage)
	w = realw99		!initial guess of wages equal the real ones from hte data

	! =======================================================================
	! read calibrated parameters, technology and lambda

	open(2, file = 'calib_param.dat', STATUS ='OLD')
		do j = 1, n
			read(2,*), x(j)
		end do
	close(2)

	d1 = x(1)
	d2 = x(2)
	d3 = x(3)
	b = x(4)
	l = x(5)
	tagree = x(6)
	alpha = x(7)		! share of type A goods
	thetab = x(8)		! theta for type B goods
	sigmab = x(9)		! sigmab must be between (thetab + 1) and 1

	open(2, file = 'calib_tech.dat', STATUS ='OLD')
		do j = 1, NN
			read(2,*), T0(j)
		end do
	close(2)
	T1 = T0

	! read initial guess of lambda
	open(2, file = 'calib_lambda.dat', STATUS ='OLD')
		do j = 1, NN
			read(2,*), lambda(j)
		end do
	close(2)

	! =======================================================================
	! construct iceberg cost matrix
	! =======================================================================
	call iceberg(Dist, NN, distance, contig, lang, agree, d1, d2, d3, b, l, tagree)

	! =========================================================================
	! output file
	open(5, file = 'output1.csv', STATUS ='REPLACE')
	open(6, file = 'output2.csv', STATUS ='REPLACE')
	open(7, file = 'output3.csv', STATUS ='REPLACE')
	open(8, file = 'output4.csv', STATUS ='REPLACE')
	open(9, file = 'output5.csv', STATUS ='REPLACE')
	open(10, file = 'output6.csv', STATUS ='REPLACE')
	open(11, file = 'output7.csv', STATUS ='REPLACE')
	open(12, file = 'output8.csv', STATUS ='REPLACE')
	open(13, file = 'output9.csv', STATUS ='REPLACE')
	open(14, file = 'output10.csv', STATUS ='REPLACE')
	open(15, file = 'output11.csv', STATUS ='REPLACE')
	open(16, file = 'output12.csv', STATUS ='REPLACE')

	write(5,*),  "shock, w0, w1"
	write(6,*),  "shock, T0, T1"
	write(7,*),  "shock, XATn0, XATn1"
	write(8,*),  "shock, XATi0, XATi1"
	write(9,*),  "shock, U0, U1"
	write(10,*), "shock, PAB0, PAB1"
	write(11,*), "shock, trade_share0, trade_share1"
	write(12,*), "shock, PA0, PA1"
	write(13,*), "shock, PB0, PB1"
	write(14,*), "shock, thetab, thetaa"
	write(15,*), "shock, sigmab, sigmaa"
	write(16,*), "shock, realw0, realw1"

countt = 0
thetaloop : do shock = 1, NN

	! =======================================================================
	! calculate demand matrix & equilibrium wages
	! =======================================================================

	gammata = DGAMMA((theta + 1. - sigmaa)/theta)	! sector A
	gammatb = DGAMMA((thetab + 1. - sigmab)/thetab)	! sector B
	
	! ============================================================================
	! TECHNOLOGIES for original wages
	! ----------------------------------------------------------------------------
	succt = .false.
	j = 0
	T1 = T0

	call findt(NN, H, wpop, realw99, Dist, T0, lambda, XT, XA, XB, alpha, succt, theta, thetab, sigmaa, sigmab, gammata, gammatb)

	do
		if ((succt) .or. (j>= maxft)) exit
		call random_number(rd)
		rd = rd/3
		T0 = rd*T0 + (1-rd)*T1
		call findt(NN, H, wpop, realw99, Dist, T0, lambda, XT, XA, XB, alpha, succt, theta, thetab, sigmaa, sigmab, gammata, gammatb)
		j = j + 1
	end do
	j = 0
	do
		if ((succt) .or. (j>= maxft)) exit
		call findt(NN, H, wpop, realw99, Dist, T0, lambda, XT, XA, XB, alpha, succt, theta, thetab, sigmaa, sigmab, gammata, gammatb)
		j = j + 1
	end do
	
	! assert that without counterfactuals, technology parameters do not change for first run
	if (countt == 0) then
		aux1 = abs(T0 - T1)/T1
		aux2 = abs(T0 - T1)
		if ( any(aux1>=1e-14) .or. any(aux2>=1e-14) ) then
			print *, "technologies changed without counterfactual"
			do imp = 1, NN
				print*, imp, T1(imp), ( T1(imp)-T0(imp) )/T1(imp)
			end do 
		endif
	endif

	! assert that without counterfactuals wages do not change
	! WAGES
	! ----------------------------------------------------------------------------
	succt = .false.
	winit = realw99
	call findw(NN, H, wpop, winit, Dist, T0, lambda, XT, XA, XB, alpha, succt, theta, thetab, sigmaa, sigmab, gammata, gammatb)
	
	aux1 = abs(realw99 - winit)/realw99
	aux2 = abs(realw99 - winit)

	if ( any(aux1>=1e-14) .or. any(aux2>=1e-14) ) then
		print *, "wages changed without counterfactual"
		do imp = 1, NN
			print*, imp, winit(imp), ( winit(imp)- realw99(imp) )
		end do 
	endif

	! =========================================================================================
	! calculate utility (per capita)
	! =======================================================================

	!total by sector
	XAn = sum(XA, 2)	! expenditures	
	XBn = sum(XB, 2)
	XAi = sum(XA, 1)	! production
	XBi = sum(XB, 1)

	! sector A: homogeneous goods
	UA = (sigmaa/(sigmaa - 1.))*lambda*XAn
	PA0 = XAn*(lambda**sigmaa)*(alpha**(-sigmaa))
	PA0 = PA0**(1/(1-sigmaa))

	! sector B: homogeneous goods
	UB = (sigmab/(sigmab - 1.))*lambda*XBn
	PB0 = XBn*(lambda**sigmab)*((1-alpha)**(-sigmab))
	PB0 = PB0**(1/(1-sigmab))

	! total/variables to be recorded
	UT0 = UA + UB
	PAB0 = PA0/PB0
	XATn0 = XAn/(XAn + XBn)
	XATi0 = XAi/(XAi + XBi)
	tradeshare0 = sum(XT,1)
	do j = 1, NN
		tradeshare0(j) = (tradeshare0(j) - XT(j,j))/tradeshare0(j)
	end do


	print *, "shock", shock

	print *, " --------------------------------------------------------------- "
	print *, "maximum errors -- original data"	
	print *, "budget constraint                           ", maxloc(abs(XAn + XBn - realw99*wpop))
	print *, "                                                   ", maxval(abs(XAn + XBn - realw99*wpop))
	aux1 = sum(XT,2)
	aux2 = sum(XT,1)	
	print *, " "
	print *, "expenditures vs. production (as a % of GDP) ", maxloc(abs(aux1 - aux2)/aux1)
	print *, "                                                    ", maxval(abs(aux1 - aux2)/aux1)
	print *, " ------------------------------------------------------------------ "
	print *, " "

	call findrealw(NN, wpop, lambda, realw99, alpha, sigmaa, sigmab, PA0, PB0, UT0, realw0)

	! =======================================================================
	! ENTER COUNTERFACTUAL HERE
	! =======================================================================
	T1 = T0
	if (shock<=9) then
		T1(shock) = T0(shock)*15.
	else if (shock<=50) then
		T1(shock) = T0(shock)*50.
	else
		T1(shock) = T0(shock)*100.
	end if

	! China's income grows by 300% and the USA's by 30%
	if (shock == 6) then
		T1(shock) = T0(shock)*7.6e10 
	else if (shock == 1) then
		T1(shock) = T0(shock)*9
	end if

	succt = .false.
	j = 0
	w = realw0
	w(shock) = 4*w(shock)
	winit = w
	do
		if ((succt) .or. (j>= maxft)) exit
		call random_number(rd)
		rd = rd/3
		w = rd*w + (1-rd)*winit
		call findw(NN, H, wpop, w, Dist, T1, lambda, XT, XA, XB, alpha, succt, theta, thetab, sigmaa, sigmab, gammata, gammatb)
		j = j + 1
	end do
	j = 0
	do
		if ((succt) .or. (j>= maxft)) exit
		call findw(NN, H, wpop, w, Dist, T1, lambda, XT, XA, XB, alpha, succt, theta, thetab, sigmaa, sigmab, gammata, gammatb)
		j = j + 1
	end do

! =========================================================================================
! calculate utility (per capita)
! =======================================================================

	!total by sector
	XAn = sum(XA, 2)	! expenditures	
	XBn = sum(XB, 2)
	XAi = sum(XA, 1)	! production
	XBi = sum(XB, 1)

	! sector A: homogeneous goods
	UA = (sigmaa/(sigmaa - 1.))*lambda*XAn
	PA1 = XAn*(lambda**sigmaa)*(alpha**(-sigmaa))
	PA1 = PA1**(1/(1-sigmaa))

	! sector B: homogeneous goods
	UB = (sigmab/(sigmab - 1.))*lambda*XBn
	PB1 = XBn*(lambda**sigmab)*((1-alpha)**(-sigmab))
	PB1 = PB1**(1/(1-sigmab))

	! total/variables to be recorded
	UT1 = UA + UB
	PAB1 = PA1/PB1
	XATn1 = XAn/(XAn + XBn)
	XATi1 = XAi/(XAi + XBi)
	tradeshare1 = sum(XT,1)
	do j = 1, NN
		tradeshare1(j) = (tradeshare1(j) - XT(j,j))/tradeshare1(j)
	end do
	
	call findrealw(NN, wpop, lambda, w, alpha, sigmaa, sigmab, PA0, PB0, UT1, realw1)

	! ============================================================================
	! record results
	! ============================================================================
!	5  "shock, w0, w1"
!	6  "shock, T0, T1"
!	7  "shock, XATn0, XATn1"
!	8  "shock, XATi0, XATi1"
!	9  "shock, U0, U1"
!	10 "shock, PAB0, PAB1"
!	12 "shock, PA0, PA1"
!	13 "shock, PB0, PB1"
!	14 "shock, thetab, thetaa"
!	15 "shock, sigmab, sigmaa"
	
	do j = 1, NN
		write (5, *), shock, ", ", realw99(j), ", ", w(j)
		write (6, *), shock, ", ", T0(j), ", ", T1(j)
		write (7, *), shock, ", ",  XATn0(j), ", ",  XATn1(j)
		write (8, *), shock, ", ", XATi0(j), ", ", XATi1(j)
		write (9, *), shock, ", ", UT0(j), ", ", UT1(j)
		write (10, *), shock, ", ", PAB0(j), ", ", PAB1(j)
		write (11, *), shock, ", ", tradeshare0(j), ", ", tradeshare1(j)
		write (12, *), shock, ", ", PA0(j), ", ", PA1(j)
		write (13, *), shock, ", ", PB0(j), ", ", PB1(j)
		write (14, *), shock, ", ", thetab, ", ", theta
		write (15, *), shock, ", ", sigmab, ", ", sigmaa
		write (16, *), shock, ", ", realw0(j), ", ", realw1(j)
	end do

! =========================================================================================
! calculate & print errors in labor market and in budget constraint
! =========================================================================================	

	print *, "maximum errors -- counterfactual"	
	print *, "budget constraint                           ", maxloc(abs(XAn + XBn - w*wpop))
	print *, "                                                    ", maxval(abs(XAn + XBn - w*wpop))
	aux1 = sum(XT,2)
	aux2 = sum(XT,1)	
	print *, " "
	print *, "expenditures vs. production (as a % of GDP) ", maxloc(abs(aux1 - aux2)/aux1)
	print *, "                                                    ", maxval(abs(aux1 - aux2)/aux1)
	print *, " ------------------------------------------------------------------ "
	print *, " "

	print *, "largest difference in wages abs(real - counter)     ", maxval(abs(w - realw99)/realw99)
	print *, "==================================================================== "	
	print *, " "	

countt = countt +1

end do thetaloop

close(5); close(6); close(7); close(8); close(9); close(10); close(11); close(12); close(13); close(14); close(15); close(16)

	CALL SYSTEM_CLOCK(COUNT=endtime)

	print*, " "
	print*, "=========================================================================="
	print*, "Time: ", DBLE(endtime - startime)/(60*DBLE(hz)), " minutes"
	print*, " "
	print*, " " 

end program main

	
!	x(8) = 1./(x(8) + 1)		! homogeneous goods share
!	x(9) = 1 + x(9)			! theta for the non-homothetic preferences
!	x(10) = (x(10)/(x(10)+1))*x(9) + 1.

!	open(2, file = 'calib_param12.dat', STATUS ='REPLACE')
!		do j = 1, n
!			write(2,*), x(j)
!		end do
!	close(2)
