! ============================================================	
!calculates objective function
! ==========================================================

subroutine calc_obj(NN, n, w, xsharea, myphia, myphib, thetaa, thetab, mylambda, x, obj)
	use MSIMSL

	implicit none

	integer(4), intent(in) :: NN, n
	real(8), dimension(n), intent(in) :: x						! parameter guess
	real(8), dimension(NN), intent(in) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN), intent(in) :: xsharea				! expenditure share of type A
	real(8), dimension(NN), intent(in) :: myphia				! prices of type A (without constant)
	real(8), dimension(NN), intent(in) :: myphib				! prices of type B (without constant)
	real(8) :: thetaa, thetab
	real(8), dimension(NN), intent(inout) :: mylambda
	real(8), intent(out) :: obj
	
	real(8), parameter :: sigmaa = 5.00
	real(8), dimension(NN) :: xsharea_hat, objbyimp	
	real(8):: gammata, gammatb, alphaa, sigmab
	integer(4) :: j
	
	
	alphaa = 1./(x(1) + 1)					! homogeneous goods share
	sigmab = (x(2)/(x(2)+1))*thetab + 1.	!sigma must be between (theta + 1) and 1

	gammata = DGAMMA((thetaa + 1. - sigmaa)/thetaa)	! type A
	gammatb = DGAMMA((thetab + 1. - sigmab)/thetab)	! type B
	
if ((gammata <= 0.) .or. (gammatb <=0.)) then; print *, "gamma function is negative"; stop; end if

	! =======================================================================
	! estimate lambdas and get estimated excess demand vector

	call ilambdas(NN, w, myphia, myphib, mylambda, alphaa, thetaa, thetab, sigmaa, sigmab, gammata, gammatb, xsharea_hat)

	! =========================================================================================
	! objective function: sum of squared errors
	obj = sum((xsharea - xsharea_hat)**2)
	obj = obj/sum(xsharea**2)

	! print to stata a file with estimated xsharea from FE and second stage
!	open(14, file = 'output_xsharea_hat.csv', STATUS ='REPLACE')
!	write(14,*), "country, xsharea, xsharea_hat"
!	do j = 1, NN
!		write(14,*), j, ", ", xsharea(j), ", ", xsharea_hat(j)
!	end do
!	close(14)


end subroutine calc_obj


! ==========================================================================
! ==========================================================================

subroutine ilambdas(NN, w, myphia, myphib, lambda, alphaa, thetaa, thetab, sigmaa, sigmab, gammata, gammatb, xsharea_hat)
	use MSIMSL
	implicit none
	
	real(8), parameter :: toll = 5e-7
	integer(4), parameter :: maxi = 5000

	integer(4), intent(in) :: NN
	real(8), intent(in) :: thetaa, sigmab, gammatb, sigmaa, alphaa, gammata, thetab
	real(8), dimension(NN), intent(in) :: myphia, myphib, w
	real(8), dimension(NN), intent(inout):: lambda
	real(8), dimension(NN), intent(out):: xsharea_hat
	
	integer(4):: imp, exp, counti, j
	real(8) :: xa, xb
	real(8) :: error, errold, spsi, rd, ctrlam, lamold, auxa, auxb

	do imp = 1, NN
		spsi = 0.5
		counti = 1
		ctrlam = lambda(imp)

		auxa = (alphaa**(sigmaa))*(myphia(imp)**(1 - sigmaa))*gammata
		auxb = ((1 - alphaa)**(sigmab))*(myphib(imp)**(1 - sigmab))*gammata

		! initial guess of lambda (if expenditures in A are greater than wealth)
		if ( ctrlam <= auxa/(w(imp))**(1/sigmaa) ) then
			if (alphaa == 1) then
				ctrlam = auxa/(w(imp)**(1/sigmaa))
			else
				ctrlam = auxa/(w(imp)**(1/sigmaa))*1.1
			end if
		end if
			

		! calculate expenditures for a given lambda
		! ================================================================
		xa = (ctrlam**(-sigmaa))*auxa			! spending in type A
		xb = (ctrlam**(-sigmab))*auxb			! spending in type B
		
		! ==================================================================
		error = xa + xb - w(imp)
		lamold = ctrlam
		errold = error

		implamb: do
			! stop controls
			!==========================================================
			if (abs(error) < toll) then
				lambda(imp) = ctrlam
				exit implamb
			else if ( counti >= maxi) then
				print *, "lambdas did not converge"
				stop
				exit implamb
			end if

			! ============================================================
			! new guess
			! ============================================================
			ctrlam = lamold*(1. + spsi*error)
			if (ctrlam < 0) then
				spsi = spsi*(1.e-1)
				ctrlam = 0.9*lamold
			end if

			! calculate expenditures for a given lambda
			! ================================================================
			xa = (ctrlam**(-sigmaa))*auxa		! spending in type A
			xb = (ctrlam**(-sigmab))*auxb		! spending in type B
		
			! ==================================================================
			error = xa + xb - w(imp)
		
			call random_number(rd)
			if (rd > (49./50.)) then
				spsi = max(min(10., spsi), 0.01)
			else if (abs(error + errold) < abs(error) + abs(errold)) then
			! decrease sensitivity if error changed sign
				spsi = spsi*(0.9 + rd/8.)
			! increase otherwise
			else 
				spsi = spsi*(1.2 + rd/8.)
			end if		

			if (abs(error) > abs(errold)) then
				error = errold
				ctrlam = lamold
			else
				lamold = ctrlam
				errold = error
			end if
			
			counti = counti + 1

		end do implamb
		xsharea_hat(imp) = xa/(xa+xb)

	end	do


end subroutine ilambdas