! ============================================================	
!calculates objective function
! ==========================================================

subroutine calc_obj(NN, n, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda, x, obj)
	use MSIMSL

	implicit none

	integer(4), intent(in) :: NN, n
	real(8), dimension(n), intent(in) :: x						! parameter guess
	integer(4), dimension(NN), intent(in) :: availability
	real(8), dimension(NN), intent(in) :: w	, w_highest, w_fourth, w_third, w_second, w_lowest					! wages 1999 (normalized to US = 1)
	real(8), dimension(NN), intent(in) :: xsharea				! expenditure share of type A
	real(8), dimension(NN), intent(in) :: myphia				! prices of type A (without constant)
	real(8), dimension(NN), intent(in) :: myphib				! prices of type B (without constant)
	real(8) :: thetaa, thetab
	real(8), dimension(NN), intent(inout) :: mylambda
	real(8), intent(out) :: obj
	
!	real(8), parameter :: sigmaa = 5.0
	real(8), dimension(NN) :: xsharea_hat, objbyimp	
	real(8):: gammata, gammatb, alphaa, sigmab, xa_aux, denominator
	integer(4) :: j
	real(8), parameter :: sigmaa = 5.d0
	
	alphaa = 1./(x(1) + 1)					! homogeneous goods share
	sigmab = (x(2)/(x(2)+1))*thetab + 1.	!sigma must be between (theta + 1) and 1

	gammata = DGAMMA((thetaa + 1. - sigmaa)/thetaa)	! sector A
	gammatb = DGAMMA((thetab + 1. - sigmab)/thetab)	! sector B
	
if ((gammata <= 0.) .or. (gammatb <=0.)) then; print *, "gamma function is negative"; stop; end if


	! =======================================================================
	! estimate lambdas and get estimated excess demand vector
	xsharea_hat = 0.d0
	do j = 1, NN
		if (availability(j) == 0) then	
			! no income inequality
			call ilambdas(w(j), myphia(j), myphib(j), mylambda(j), alphaa, thetaa, thetab, sigmaa, sigmab, gammata, gammatb, xa_aux)
			xsharea_hat(j) = xa_aux/w(j)

		else
			call ilambdas(w_highest(j), myphia(j), myphib(j), mylambda(j), alphaa, thetaa, thetab, sigmaa, sigmab, gammata, gammatb, xa_aux)
			xsharea_hat(j) = xa_aux

			call ilambdas(w_fourth(j), myphia(j), myphib(j), mylambda(j), alphaa, thetaa, thetab, sigmaa, sigmab, gammata, gammatb, xa_aux)
			xsharea_hat(j) = xsharea_hat(j) + xa_aux
			
			call ilambdas(w_third(j), myphia(j), myphib(j), mylambda(j), alphaa, thetaa, thetab, sigmaa, sigmab, gammata, gammatb, xa_aux)
			xsharea_hat(j) = xsharea_hat(j) + xa_aux

			call ilambdas(w_second(j), myphia(j), myphib(j), mylambda(j), alphaa, thetaa, thetab, sigmaa, sigmab, gammata, gammatb, xa_aux)
			xsharea_hat(j) = xsharea_hat(j) + xa_aux
			
			call ilambdas(w_lowest(j), myphia(j), myphib(j), mylambda(j), alphaa, thetaa, thetab, sigmaa, sigmab, gammata, gammatb, xa_aux)
			xsharea_hat(j) = xsharea_hat(j) + xa_aux

			xsharea_hat(j) = xsharea_hat(j)/(w_highest(j) + w_fourth(j) + w_third(j) + w_second(j) + w_lowest(j))

		end if
	end do

	! =========================================================================================
	! objective function: sum of squared errors
	obj = sum((xsharea - xsharea_hat)**2)
	obj = obj/sum(xsharea**2)


	! =========================================================================================
	! objective function: for countries with available data only
	obj = 0.d0
	denominator = 0.d0
	do j = 1, NN
		if (availability(j) == 1) then
			obj = obj + (xsharea(j) - xsharea_hat(j))**2
			denominator = denominator + (xsharea(j)**2)
		end if
	end do
	obj = obj/denominator

end subroutine calc_obj


! ==========================================================================
! ==========================================================================

subroutine ilambdas(w, myphia, myphib, ctrlam, alphaa, thetaa, thetab, sigmaa, sigmab, gammata, gammatb, xa)
	use MSIMSL
	implicit none
	
	real(8), parameter :: toll = 5e-7
	integer(4), parameter :: maxi = 5000

	real(8), intent(in) :: thetaa, sigmab, gammatb, sigmaa, alphaa, gammata, thetab
	real(8), intent(in) :: myphia, myphib, w
	real(8), intent(inout):: ctrlam
	real(8), intent(out):: xa
	
	integer(4):: imp, exp, counti, j
	real(8) :: xb
	real(8) :: error, errold, spsi, rd, lamold, auxa, auxb

	! find lambda
		spsi = 0.5
		counti = 1

		auxa = (alphaa**(sigmaa))*(myphia**(1 - sigmaa))*gammata
		auxb = ((1 - alphaa)**(sigmab))*(myphib**(1 - sigmab))*gammata

		! initial guess of lambda (if expenditures in A are greater than wealth)
		if ( ctrlam <= auxa/(w)**(1/sigmaa) ) then
			if (alphaa == 1) then
				ctrlam = auxa/(w**(1/sigmaa))
			else
				ctrlam = auxa/(w**(1/sigmaa))*1.1
			end if
		end if
			

		! calculate expenditures for a given lambda
		! ================================================================
		xa = (ctrlam**(-sigmaa))*auxa			! spending in type A
		xb = (ctrlam**(-sigmab))*auxb			! spending in type B
		
		! ==================================================================
		error = xa + xb - w
		lamold = ctrlam
		errold = error
	
		implamb: do
			! stop controls
			!==========================================================
			if (abs(error) < toll) then
				exit implamb
			else if ( counti >= maxi) then
				print *, "lambdas did not converge"
				stop
				exit implamb
			end if

			! ============================================================
			! new guess
			! ============================================================
			ctrlam = lamold*(1. + spsi*error)
			if (ctrlam < 0) then
				spsi = spsi*(1.e-1)
				ctrlam = 0.9*lamold
			end if

			! calculate expenditures for a given lambda
			! ================================================================
			xa = (ctrlam**(-sigmaa))*auxa		! spending in type A
			xb = (ctrlam**(-sigmab))*auxb		! spending in type B
		
			! ==================================================================
			error = xa + xb - w
		
			call random_number(rd)
			if (rd > (49./50.)) then
				spsi = max(min(10., spsi), 0.01)
			else if (abs(error + errold) < abs(error) + abs(errold)) then
			! decrease sensitivity if error changed sign
				spsi = spsi*(0.9 + rd/8.)
			! increase otherwise
			else 
				spsi = spsi*(1.2 + rd/8.)
			end if		

			if (abs(error) > abs(errold)) then
				error = errold
				ctrlam = lamold
			else
				lamold = ctrlam
				errold = error
			end if
			
			counti = counti + 1

		end do implamb

end subroutine ilambdas