! ============================================================	
!calculates objective function
! ==========================================================

subroutine calc_obj(NN, fiveNN, n, H, w, wpop, XT99, distance, contig, lang, agree, uswage, w_highest, w_fourth, w_third, w_second, w_lowest, T, lambda, succt, x, obj)
		
	use MSIMSL

	implicit none

	integer(4), intent(in) :: NN, fiveNN, n
	real(8), dimension(n), intent(in) :: x						! parameter guess
	real(8), dimension(NN), intent(in) :: H						! population	in millions
	real(8), dimension(NN), intent(in) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN), intent(in) :: wpop					! working population (as a faction of total)
	real(8), dimension(NN, NN), intent(in) :: XT99				! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN), intent(in) :: distance			! distance in km
	integer(4), dimension(NN, NN), intent(in) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN), intent(in) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN), intent(out) :: agree			! 1 for both countries in the same trade agreement--ASEAN, COMESA, EFTA, MERCOSUR, NAFTA, EU
	real(8), intent(in) :: uswage
	real(8), dimension(NN), intent(in) :: w_highest, w_fourth, w_third, w_second, w_lowest
	
	real(8), dimension(NN), intent(inout) :: T
	real(8), dimension(fiveNN), intent(inout) :: lambda
	logical, intent(out) :: succt
	real(8), intent(out) :: obj
	
	integer(4), parameter :: maxft = 20

	real(8) :: d1, d2, d3, b, l, teu, tagree, rd, gammatb, gammata, thetab , alpha, sigmab, denominator, numerator, npos, nneg, ntotal, residual
	real(8), dimension(NN) :: tinit, gdp		!objperc is the objective function per country
	real(8), dimension(NN, NN) :: Dist, XT
	integer(4) :: j, imp, exp
	
	integer(4), dimension(NN, NN) :: aux
	real(8), parameter :: theta = 8.28, &				!8.28, 3.60 and 12.86 are the estimates in EK
						  sigmaa = 5.0, &				! Broda and Weinstein
						  weight = 2.0

	d1 = x(1)
	d2 = x(2)
	d3 = x(3)
	b = x(4)
	l = x(5)
	tagree = x(6)
	alpha = 1./(x(7) + 1)		! homogeneous goods share
	thetab = 1 + x(8)			! theta for type B
	sigmab = (x(9)/(x(9)+1))*thetab + 1.	!sigmaa must be between (theta + 1) and 1

	gammata = DGAMMA((theta + 1. - sigmaa)/theta)	! type A
	gammatb = DGAMMA((thetab + 1. - sigmab)/thetab)	! type B

	! =======================================================================
	! construct iceberg cost matrix

	call iceberg(Dist, NN, distance, contig, lang, agree, d1, d2, d3, b, l, tagree)

	! =======================================================================
	! calculate trade flows

	succt = .false.
	j = 0
	tinit = T
	do
		if ((succt) .or. (j>= maxft)) exit
		call random_number(rd)
		rd = rd/3
		T = rd*T + (1-rd)*tinit
		call findt(NN, fiveNN, H, wpop, w, w_highest, w_fourth, w_third, w_second, w_lowest, Dist, T, lambda, XT, alpha, succt, theta, thetab, sigmaa, sigmab, gammata, gammatb)
		j = j + 1
	end do
	j = 0
	do
		if ((succt) .or. (j>= maxft)) exit
		call findt(NN, fiveNN, H, wpop, w, w_highest, w_fourth, w_third, w_second, w_lowest, Dist, T, lambda, XT, alpha, succt, theta, thetab, sigmaa, sigmab, gammata, gammatb)
		j = j + 1
	end do

	do j = 1, NN
		XT(j,j) = 0
	end do

	XT = XT*uswage

! =========================================================================================
! objective function

	gdp = uswage*w*wpop*H
	obj = 0.
	denominator = 0.
	j = 0
	do imp = 1, NN
		do exp = 1, NN
		if ( (imp/=exp) .and. (XT99(imp,exp) >= -.0000001)  ) then
			obj = obj + ((XT99(imp,exp) - XT(imp,exp))**2)/((gdp(imp)*gdp(exp))**weight)
			denominator = denominator + ((XT99(imp,exp))**2)/((gdp(imp)*gdp(exp))**weight)
			j = j + 1
		end if
		end do
	end do
	obj = obj/denominator


end subroutine calc_obj
