! ============================================================	
!calculates objective function
! ==========================================================

subroutine calc_optimal(NN, n, nobs, theta, sigmaa, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, lambda, succt, x, obj, XTdata, XToptimal, residuals, obsimpexp)

	use MSIMSL

	implicit none

	integer(4), intent(in) :: NN, n, nobs
	real(8), dimension(n), intent(in) :: x						! parameter guess
	real(8), dimension(NN), intent(in) :: H						! population	in millions
	real(8), dimension(NN), intent(in) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN), intent(in) :: wpop					! working population (as a faction of total)
	real(8), dimension(NN, NN), intent(in) :: XT99				! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN), intent(in) :: distance			! distance in km
	integer(4), dimension(NN, NN), intent(in) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN), intent(in) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN), intent(in) :: agree			! 1 for both countries in the same trade agreement--ASEAN, COMESA, EFTA, MERCOSUR, NAFTA, EU
	real(8), intent(in) :: uswage, theta, sigmaa

	real(8), dimension(NN), intent(inout) :: T, lambda
	real(8), dimension(nobs), intent(out) :: XTdata, XToptimal, residuals
	integer(4), dimension(nobs, 2), intent(out) :: obsimpexp
	logical, intent(out) :: succt
	real(8), intent(out) :: obj
	
	integer(4), parameter :: maxft = 20

	real(8) :: d1, d2, d3, b, l, teu, tagree, gammatb, gammata, thetab , alpha, sigmab, denominator, numerator, npos, nneg, residual
	real(8), dimension(NN) :: gdp
	real(8), dimension(NN, NN) :: Dist, XT
	integer(4) :: j, imp, exp, titer
	
	integer(4), dimension(NN, NN) :: aux
					  		

	d1 = x(1)
	d2 = x(2)
	d3 = x(3)
	b = x(4)
	l = x(5)
	tagree = x(6)
	alpha = 1./(x(7) + 1)		! homogeneous goods share
	thetab = 1 + x(8)			! theta for type B
	sigmab = (x(9)/(x(9)+1))*thetab + 1.	!sigmaa must be between (theta + 1) and 1

	gammata = DGAMMA((theta + 1. - sigmaa)/theta)	! type A
	gammatb = DGAMMA((thetab + 1. - sigmab)/thetab)	! type B

!print *, "alpha", alpha
!print *, "sigmaa", sigmaa
!print *, "sigmab", sigmab
!print *, "thetaa", theta
!print *, "thetab", thetab
!print *, "gammatb", gammatb
!print *, "gammata", gammata
!print *, " =============================== "
!print *, " "

	! =======================================================================
	! construct iceberg cost matrix

	call iceberg(Dist, NN, distance, contig, lang, agree, d1, d2, d3, b, l, tagree)

	! =======================================================================
	! calculate excess demand vector

	succt = .false.
	j = 0
	call findt(NN, H, wpop, w, Dist, T, lambda, XT, alpha, succt, theta, thetab, sigmaa, sigmab, gammata, gammatb, titer)

	do j = 1, NN
		XT(j,j) = 0
	end do

	XT = XT*uswage


! =========================================================================================
! objective function

	gdp = uswage*w*wpop*H
	obj = 0.
	denominator = 0.
	j = 0
	do imp = 1, NN
		do exp = 1, NN
		if ( (imp/=exp) .and. (XT99(imp,exp) >= -.0000001)  ) then
			obj = obj + ((XT99(imp,exp) - XT(imp,exp))**2)/((gdp(imp)*gdp(exp))**2)
			denominator = denominator + ((XT99(imp,exp))**2)/((gdp(imp)*gdp(exp))**2)
			j = j + 1
			XTdata(j) = XT99(imp, exp)/(gdp(imp)*gdp(exp))
			XToptimal(j) = XT(imp, exp)/(gdp(imp)*gdp(exp))
			residuals(j) = XTdata(j) - XToptimal(j)
			obsimpexp(j,:) = (/ imp, exp /)
		end if
		end do
	end do
	obj = obj/denominator
	print *, " ----------------------------------- "
	print *, "tests below must equal zero"
	print *, "number of observations: ", j - nobs
	print *, "objective function:              ", obj - sum(residuals**2)/denominator
	print *, " ----------------------------------- "
	print *, "objective function = ", obj
	print *, " "

! print output to calculate clustered standard errors in Stata--this program was to be activated only before program to calculate cluster-robust std. error was written in fortran
!call cluster_output(nobs, n, XTdata, obsimpexp, residuals)

end subroutine calc_optimal


! ===================================================================
subroutine cluster_output(nobs, n, XTdata, obsimpexp, residuals)
	implicit none

	integer(4), intent(in) :: nobs, n
	real(8), dimension(nobs), intent(in) :: XTdata, residuals
	integer(4), dimension(nobs, 2) :: obsimpexp

	integer :: j

	! ------------------------------
	open(1, file = 'obstradeflows.csv', STATUS ='REPLACE')
	open(2, file = 'obsimpexp.csv', STATUS ='REPLACE')
	open(3, file = 'residuals.csv', STATUS ='REPLACE')
	write(1,*), "nobs, tradeflow"
	write(2,*), "nobs, importer, exporter"
	write(3,*), "nobs, residuals"

	do j = 1, nobs
		write(1,*), j, ", ", XTdata(j)
		write(2,*), j, ", ", obsimpexp(j, 1), ", ", obsimpexp(j,2)
		write(3,*), j, ", ", residuals(j)
	end do 
	close(1)
	close(2)
	close(3)

end subroutine