! ============================================================	
! estimates the derivative of the objective function with respect to each parameter
! ==========================================================

subroutine calc_derivatives(NN, n, nobs, theta, H, w, wpop, XT99, distance, contig, lang, agree, uswage, Toptimal, XToptimal, xoptimal, Xderivatives)
	use MSIMSL

	implicit none

	integer(4), intent(in) :: NN, n, nobs
	real(8), dimension(n), intent(in) :: xoptimal				! parameter at optimal
	real(8), dimension(NN), intent(in) :: H						! population	in millions
	real(8), dimension(NN), intent(in) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN), intent(in) :: wpop					! working population (as a faction of total)
	real(8), dimension(NN, NN), intent(in) :: XT99				! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN), intent(in) :: distance			! distance in km
	integer(4), dimension(NN, NN), intent(in) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN), intent(in) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN), intent(in) :: agree			! 1 for both countries in the same trade agreement--ASEAN, COMESA, EFTA, MERCOSUR, NAFTA, EU
	real(8), intent(in) :: uswage, theta

	real(8), dimension(NN), intent(in) :: Toptimal				! T at xoptimal
	real(8), dimension(nobs), intent(in) :: XToptimal			! trade flows as predicted by the model
	real(8), dimension(nobs, n), intent(out) :: Xderivatives
		
	
	integer(4), parameter :: maxft = 20
	logical :: succt
	real(8):: obj
	real(8), dimension(n):: hjump_perc, xoptimal2, xplush
	real(8), dimension(nobs) :: XTxplush

	real(8) :: d1, d2, d3, b, l, teu, tagree, rd, thetab, denominator, numerator, hjump
	real(8), dimension(NN) :: xshareA
	real(8), dimension(NN) :: tinit, T, gdp
	real(8), dimension(NN, NN) :: Dist, XT
	integer(4) :: j, jj, imp, exp, titer

	thetab = 1 + xoptimal(7)							! theta for type B > 1
	xshareA = xoptimal(n-NN+1:n)/(1+xoptimal(n-NN+1:n))		! share of type A in spending must be between zero and 1
	xoptimal2 = xoptimal
	xoptimal2(7) = thetab
	xoptimal2(n-NN+1:n) = xshareA

	! size of derivative jump as a multiple of original parameter value -- the number of iterations for each parameter should be between 50 and 150.
	hjump_perc = 1.001
			
! ============================================================================
! main loop : find derivative vector for each parameter guess

parameter_loop: do jj = 1, n

	! this loops asserts that the number of iterations to find T is between 1 and 80 for xshareA parameters
	titer = 0
	within_loop: do
			
	xplush = xoptimal2
	xplush(jj) = xoptimal2(jj)*hjump_perc(jj)
	if ((jj > n - NN) .and. (xoptimal2(jj) > .5)) xplush(jj) = xoptimal2(jj)*(2. - hjump_perc(jj))

! the lines below specify which parameters are log-normally distributed, and which are not.
	if (jj == 7) then
		hjump = ( log(xplush(jj) - 1) - log(xoptimal2(jj) - 1) )
	elseif ((jj == 1) .or. (jj == 4)  .or. (jj == 5)  .or. (jj == 6)) then
		hjump = ( log(xplush(jj)) - log(xoptimal2(jj)) )
	elseif (jj > 7) then
!		print *,jj, hjump_perc(108), xplush(jj), xoptimal2(jj) 
		hjump = ( log(xplush(jj)/(1 - xplush(jj)) ) - log(xoptimal2(jj) / (1 - xoptimal2(jj)) ) )	
	else												! free parameters
		hjump = xplush(jj) - xoptimal2(jj)
	end if 


	! assign new parameter values
	d1 = xplush(1)
	d2 = xplush(2)
	d3 = xplush(3)
	b = xplush(4)
	l = xplush(5)
	tagree = xplush(6)
	thetab = xplush(7)
	xshareA = xplush(n-NN+1:n)	

	! =======================================================================
	! construct iceberg cost matrix

	call iceberg(Dist, NN, distance, contig, lang, agree, d1, d2, d3, b, l, tagree)

	! =======================================================================
	! calculate excess demand vector
	T = Toptimal
	succt = .false.
	j = 0
	tinit = T
	do
		if ((succt) .or. (j>= maxft)) exit
		call random_number(rd)
		rd = rd/3
		T = rd*T + (1-rd)*tinit
		call findt(NN, H, wpop, w, Dist, T, XT, succt, theta, thetab, xshareA, titer)
		j = j + 1
	end do
	j = 0
	do
		if ((succt) .or. (j>= maxft)) exit
		call findt(NN, H, wpop, w, Dist, T, XT, succt, theta, thetab, xshareA, titer)
		print *, "titer", titer
		j = j + 1
	end do

	do j = 1, NN
		XT(j,j) = 0
	end do

	XT = XT*uswage


! =========================================================================================
! objective function

	gdp = uswage*w*wpop*H
	obj = 0.
	denominator = 0.
	j = 0
	do imp = 1, NN
		do exp = 1, NN
		if ( (imp/=exp) .and. (XT99(imp,exp) >= -.0000001)  ) then
			obj = obj + ((XT99(imp,exp) - XT(imp,exp))**2)/((gdp(imp)*gdp(exp))**2.)
			denominator = denominator + ((XT99(imp,exp))**2)/((gdp(imp)*gdp(exp))**2.)
			j = j + 1
			XTxplush(j) = XT(imp, exp)/(gdp(imp)*gdp(exp))
		end if
		end do
	end do
	obj = obj/denominator
	print *, "parameter, hjump, titer, obj", jj, hjump_perc(jj), titer, obj
	print *, " "
		! stop control
		call random_number(rd)
		if (jj > n-NN) then
			if ( (titer > 1) .and. (titer < 80) )then
			exit within_loop
			elseif (titer > 100) then
			hjump_perc(jj) = (hjump_perc(jj) - 1.)/(1.2 + rd) + 1.
			else
			hjump_perc(jj) = (hjump_perc(jj) - 1.)*(1.2 + rd/2.) + 1.
			end if
!			print *, "hjump_perc increase", hjump_perc(jj)
		else
			if ( (titer > 50) .and. (titer <100) ) then
			exit within_loop
			elseif (titer > 100) then
			hjump_perc(jj) = (hjump_perc(jj) - 1.)/(1.2 + rd) + 1.
			else
			hjump_perc(jj) = (hjump_perc(jj) - 1.)*(1.2 + rd/2.) + 1.
			end if
!			print *, "hjump_perc decrease", hjump_perc(jj)
		end if
	
	
	end do within_loop

	Xderivatives(:, jj)	= (XToptimal - XTxplush)/hjump
	
	
end do parameter_loop

end subroutine calc_derivatives
