!     findt.F90      calculates the iceberg cost matrix

subroutine findt(NN, H, wpop, w, Dist, T, lambda, XT, XA, XB, alpha, succt, theta, thetab, sigmaa, sigmab, gammata, gammatb)
		        
	use MSIMSL
	implicit none
	
	integer(4), intent(in) :: NN
	real(8), intent(in) :: theta, gammatb, gammata, sigmaa, sigmab, alpha, thetab
	real(8), dimension(NN, NN), intent(in) :: Dist
	real(8), dimension(NN), intent(in) :: H, wpop, w

	real(8), dimension(NN), intent(inout) :: T, lambda
	real(8), dimension(NN, NN), intent(out) :: XT, XA, XB
	logical, intent(out) :: succt	
	
	real(8), parameter :: tolt = .5e-5					! tolerance for larger countries is a fraction of this value
	integer(4), parameter :: maxiter = 400
	real(8), parameter :: itsi = 0.03, errh = 0.3		! initial sensitivity of T to excess demand

	integer(4):: j, realiter, iter						!counters
	real(8), dimension(NN, NN):: LT
	real(8), dimension(NN):: vtsi, TH, thmin, thold, zpmin, zpold, aux, z, zp
	real(8) :: error, errmin, errold, perc, rd, rd2
	
	! =================================================================
	! set initial sensitivity parameters and reset counters
	vtsi = itsi				! vector of sensitivity parameters
	iter = 0
	realiter = 0
	T = T/T(1)
	TH = T/H

	! ==================================================================

	call excdemand(NN, H, wpop, w, Dist, T, lambda, XT, XA, XB, LT, z, zp, alpha, theta, thetab, sigmaa, sigmab, gammata, gammatb)
				  
	error = maxval(abs(zp*(H**(errh))))
	thmin = TH
	errmin = error
	zpmin = zp

	where(abs(zp*(H**(errh))) > tolt)
		aux = 1
	elsewhere
		aux = 0
	end where
	
	! ======================================================================
	! main loop
		
	findt_loop: do
		
		! STOP COUNTROLS
		! =======================================================================================
		succt = (error < tolt)
		if (succt) then
			exit findt_loop
		else if ((realiter >= maxiter * 3) .or. (iter >= maxiter)) then
			succt = .false.
			exit findt_loop
		end if
		thold = TH
		errold = error
		zpold = zp

		! NEW GUESS
		! ==========================================================================================	

		perc = 1/(sum(aux)**.5)	! if a lot of countries need to adjust, the percentage of adjustment must be small
		where (aux == 1) 
			TH = max(min(thold*(1 - vtsi*zp), (1 + perc)*thold), max(1 - perc, 0.5)*thold)
		elsewhere
			TH = max(min(thold*(1 - 0.5*vtsi*zp), 1.015*thold), 0.985*thold)		! changes at most 1.5%
		end where
		T = TH*H
		T = T/T(1)

		! NEW EXECUTION
		! ==========================================================================================
		call excdemand(NN, H, wpop, w, Dist, T, lambda, XT, XA, XB, LT, z, zp, alpha, theta, thetab, sigmaa, sigmab, gammata, gammatb)

		error = maxval(abs(zp*(H**(errh))))

		if (error < errmin) then
			errmin = error
			thmin = TH
			zpmin = zp
		end if

		where(abs(zp*(H**(errh))) > tolt)
			aux = 1
		elsewhere
			aux = 0
		end where

		if (error > 50) then
			rd = 1
			iter = int(iter/3) + 2
		else
			call random_number(rd)
		end if
		
		if (rd > 49./50.) then
		! reset all values
			vtsi = max(min(10., vtsi), 0.01)
			error = errmin
			TH = thmin
			zp = zpmin
		!INSERIR TEXTO 1 AQUI
		else if (error >= 1.1*errold) then
			call random_number(rd2)
			! decrease sensitivity of countries whose error changed signs
			where ( ( ((zp>0) .and. (zpold<0)) .or. ((zp<0) .and. (zpold>0)) ) .and. (aux == 1) )
				vtsi = (0.95 - rd2/10)*vtsi
			end where
			! increase sensitivity of countries whose error did not changed signs
			where ( ( ((zp>0) .and. (zpold>0)) .or. ((zp<0) .and. (zpold<0))) .and. (aux == 1) )
				vtsi = (1.05 + rd2/10)*vtsi
			end where
		else if (rd < 0.3) then
			call random_number(rd2)
			! decrease sensitivity of countries whose error changed signs
			where ( ( ((zp>0) .and. (zpold<0)) .or. ((zp<0) .and. (zpold>0)) ) .and. (aux == 1) )
					vtsi = (0.95 - rd2/20)*vtsi
			end where
			! increase sensitivity of countries whose error did not changed signs
			where ( ( ((zp>0) .and. (zpold>0)) .or. ((zp<0) .and. (zpold<0)) ).and. (aux == 1) )
					vtsi = (1.05 + rd2/20)*vtsi
			end where

		end if

		realiter = realiter + 1
		iter = iter + 1
				
	end do findt_loop

end subroutine findt
