! ============================================================	
! prints to csv files variables that are used in estimating demand parameters in the second stage
! ==========================================================

subroutine print_prices(NN, n, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, succt, x, obj)
		
	use MSIMSL

	implicit none

	integer(4), intent(in) :: NN, n
	real(8), dimension(n), intent(in) :: x						! parameter guess
	real(8), dimension(NN), intent(in) :: H						! population	in millions
	real(8), dimension(NN), intent(in) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN), intent(in) :: wpop					! working population (as a faction of total)
	real(8), dimension(NN, NN), intent(in) :: XT99				! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN), intent(in) :: distance			! distance in km
	integer(4), dimension(NN, NN), intent(in) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN), intent(in) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN), intent(out) :: agree			! 1 for both countries in the same trade agreement--ASEAN, COMESA, EFTA, MERCOSUR, NAFTA, EU
	real(8), intent(in) :: uswage

	real(8), dimension(NN), intent(inout) :: T
	logical, intent(out) :: succt
	real(8), intent(out) :: obj
	
	integer(4), parameter :: maxft = 20

	real(8) :: d1, d2, d3, b, l, teu, tagree, rd, thetab, denominator, numerator, npos, nneg, residual
	real(8), dimension(NN) :: xshareA, yshareA
	real(8), dimension(NN) :: gdp, cgdp, auxa, auxb, numeratora, numeratorb, pricea, priceb
	real(8), dimension(NN, NN) :: Dist, XT, XA
	integer(4) :: j, imp, exp
	
	integer(4), dimension(NN, NN) :: aux
	real(8), parameter :: theta = 8.28, &				!8.28, 3.60 and 12.86 are the estimates in EK
						  weight = 2.0				

	d1 = x(1)
	d2 = x(2)
	d3 = x(3)
	b = x(4)
	l = x(5)
	tagree = x(6)
	thetab = 1 + x(7)							! theta for sector B > 1
	xshareA = x(n-NN+1:n)/(1+x(n-NN+1:n))		! share of type A in spending must be between zero and 1

	! =======================================================================
	! construct matrix of trade costs

	call iceberg(Dist, NN, distance, contig, lang, agree, d1, d2, d3, b, l, tagree)

	! =======================================================================
	! calculate prices and excess demand vector
	
	! -----------------------------------------
	! expenditure shares
	auxa = T*(w**(-theta))
	auxb = T*(w**(-thetab))
	do imp = 1, NN
		numeratora = auxa*(Dist(imp,:)**(-theta))
		numeratorb = auxb*(Dist(imp,:)**(-thetab))
		! trade shares = weighted average of type A shares and type B shares
		XT(imp,:) = xsharea(imp)*(numeratora/sum(numeratora)) + (1. - xsharea(imp))*(numeratorb/sum(numeratorb)) 
		! total trade flows = trade shares * importer total income 
		XT(imp,:) = XT(imp,:)*(w(imp)*wpop(imp)*H(imp))
		XA(imp,:) = xsharea(imp)*(numeratora/sum(numeratora))*(w(imp)*wpop(imp)*H(imp))
		! prices of type A and type B goods
		pricea(imp) = (sum(numeratora))**(-1/theta)
		priceb(imp) = (sum(numeratorb))**(-1/thetab)
	end do
	
	! consumption shares: assert calculated spending shares on A are equal to inputed
	yshareA = sum(XA,2)/sum(XT,2)
	print *,  sum(abs(yshareA - xshareA) ), " = teste"

	! production shares
	yshareA = sum(XA,1)/sum(XT,1)
	
	! other variables of interest
	gdp = uswage*w*wpop*H
	cgdp = uswage*w*wpop

! ====================================================================================
! print output files--for stata


	open(12, file = 'output_prices1.csv', STATUS ='REPLACE')
	open(13, file = 'output_prices2.csv', STATUS ='REPLACE')
	open(14, file = 'output_prices3.csv', STATUS ='REPLACE')
	open(15, file = 'output_prices4.csv', STATUS ='REPLACE')

	write(12,*), "country, pricea, priceb"
	write(13,*), "country, cgdp, gdp"
	write(14,*), "country, ysharea, xsharea"
	write(15,*), "country, w"

	do imp = 1, NN
		write(12,*), imp, ", ", pricea(imp), ", ", priceb(imp)
		write(13,*), imp, ", ", cgdp(imp), ", ", gdp(imp)
		write(14,*), imp, ", ", ysharea(imp), ", ", xsharea(imp)
		write(15,*), imp, ", ", w(imp)
	end do

	close(12); close(13); close(14); close(15)


! ====================================================================================
! print output files--for second stage of estimation in Fortran


	open(16, file = 'myphia.dat', STATUS ='REPLACE')
	open(17, file = 'myphib.dat', STATUS ='REPLACE')
	open(18, file = 'xsharea.dat', STATUS ='REPLACE')
	
	do imp = 1, NN
		write(16,*), pricea(imp)
		write(17,*), priceb(imp)
		write(18,*), xsharea(imp)
	end do

	close(16); close(17); close(18)


! =========================================================================================
! trade flows in millions of US$ -- obj as suggested by Matt Wiswall

	do j = 1, NN
		XT(j,j) = 0
	end do
	XT = XT*uswage

	gdp = uswage*w*wpop*H
	obj = 0.
	denominator = 0.
	j = 0
	do imp = 1, NN
		do exp = 1, NN
		if ( (imp/=exp) .and. (XT99(imp,exp) >= -.0000001)  ) then
			obj = obj + ((XT99(imp,exp) - XT(imp,exp))**2)/((gdp(imp)*gdp(exp))**weight)
			denominator = denominator + ((XT99(imp,exp))**2)/((gdp(imp)*gdp(exp))**weight)
			j = j + 1
		end if
		end do
	end do
	obj = obj/denominator

	print *, obj, " = obj in print_prices file"

end subroutine print_prices
