!     polytope.F90      minimizes a function using the polytope method

! ============================================================	
! ============================================================	
! START POLYTOPE EXECUTION
! ==========================================================
! ==========================================================

subroutine polytope(n, np1, two, usual_delta, zero_term_delta, prho, pchi, ppsi, psigma, tol_fun, &
					tol_par, ptol_fun, ptol_par, maxfun, maxiter, maxj, xinitial, v, fv, &
					NN, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda)

	use MSIMSL
	implicit none
	
	! POLYTOPE VARIABLES
	! =================================================================================	
	integer(4), intent(in) :: n			!number of variables to choose
	integer(4), intent(in) :: np1	
	integer(4), intent(in) :: two	
	real(8), intent(in) :: usual_delta, zero_term_delta, prho, pchi, ppsi, psigma, &
						tol_fun, tol_par, ptol_fun, ptol_par
	integer(4), intent(in) :: maxfun, maxiter
	integer(4), intent(in) :: maxj
	integer(4) :: func_evals, itercount
	real(8), dimension(n), intent(in) :: xinitial
	real(8), dimension(n) :: x, xin
	real(8), dimension(n, np1), intent(out) :: v
	real(8), dimension(np1), intent(out) :: fv

	real(8) :: fvguess				!accumulates results from each polytope
	real(8), dimension(n) :: vguess
	integer(4), dimension(np1) :: permute	!permutation matrix
	character (len = 30):: how, success
	character (len = 40):: out_success
	integer(4):: j		!counts how many times polytope has been run


	! MODEL VARIABLES/DATA/PARAMETERS
	! ==================================================================================

	integer(4), intent(in) :: NN
	real(8), dimension(NN), intent(in) :: w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib
	integer(4), dimension(NN), intent(in) :: availability
	real(8), intent(in) :: thetaa, thetab
	real(8), dimension(NN), intent(inout) :: mylambda
		
	! ============================================================	
	! SET INITIAL VALUES
	! ==========================================================
	x = xinitial
	xin = xinitial
	v = 0
	fv = 0

	! =================================================================
	! RUN FIRST POLYTOPE
	! =================================================================

	!calculate function and store results with initial guess
	v(:, 1) = xin
	call calc_obj(NN, n, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda, x, fv(1))

	! construct simplex
	call ctr_simplex(n, xin, usual_delta, zero_term_delta, v, fv, &
					NN, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda)
	
	! sort results
	call reset_permute(np1, permute)
	call DSVRGP(np1, fv, fv, permute)		!sorts fv returning both sorted vector fv and permutation vector
	call DPERMA(n, np1, v, n, permute, two, v, n)	!permutates the columns of v according to the sorting of fv

	! ======================================================================================
	! SAVE RESULTS

	call calc_obj(NN, n, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda, v(:,1), fv(1))
	
	! =======================================================================================

	! run polyloop
	how = "initial"
	itercount = 1
	func_evals = np1
	
	call poly_loop(n, np1, two, prho, pchi, ppsi, psigma, tol_fun, tol_par, maxfun, &
					maxiter, func_evals, itercount, v, fv, how, success, &
					NN, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda)
	j = 1

	! =======================================================================================

	vguess = v(:, 1)
	fvguess = fv(1)

	! =================================================================
	! CYCLE POLYTOPE (RUN IT REPEATEDLY)
	! =================================================================

	repeat_poly: do
		! =============================================================
		! STOP COUNTROLS
		! =============================================================
		! NOTE: if (fvguess < fv(1)), the program also stops
		if ((j > 1) .and. (((abs(fvguess - fv(1)))<= ptol_fun) .and. (maxval(abs(v(:, 1) - vguess))<= tol_par)) .or. (fvguess < fv(1))) then
			out_success = "across polytope minimum found"
			exit repeat_poly
		else if (j >= maxj) then
			out_success = "across polytope minimum not found"
			exit repeat_poly
		end if

		! =============================================================
		! PRINT RESULTS AFTER EACH POLYTOPE 
		! =============================================================

!			print *, "number of times polytope was run", j
!			print *, "v(:, 1)"
!			print "(f15.5, f15.5, f15.5, f15.5)", v(:, 1)
!			print *, "fv(1)"
!			print "(f15.5)", fv(1)
		
		! ============================================================	
		! SET INITIAL VALUES
		! ==========================================================
		vguess = v(:, 1)
		fvguess = fv(1)

		xin = v(:, 1)		!initial guess
		x = xin
		v = 0
		fv = 0
		
		v(:, 1) = vguess
		fv(1) = fvguess

		! DO POLYTOPE
		! construct simplex
		call ctr_simplex(n, xin, usual_delta, zero_term_delta, v, fv,  &
						NN, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda)
		
		! sort results
		call reset_permute(np1, permute)
		call DSVRGP(np1, fv, fv, permute)				!sorts fv returning both sorted vector fv and permutation vector
		call DPERMA(n, np1, v, n, permute, two, v, n)	!permutates the columns of v according to the sorting of fv

		! run polyloop
		how = "initial"
		itercount = 1
		func_evals = np1
		call poly_loop(n, np1, two, prho, pchi, ppsi, psigma, tol_fun, &
						tol_par, maxfun, maxiter, func_evals, itercount, v, fv, how, success, &
						NN, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda)
		j =	j + 1
		! ======================================================================================
		! SAVE RESULTS

		call calc_obj(NN, n, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda, v(:,1), fv(1))
		
		! =======================================================================================

	end do repeat_poly
	print *, "number of times polytope was run", j
!	print *, "v(:, 1) after several polytope loops"
!	print "(f15.5, f15.5, f15.5, f15.5)", v(:, 1)
!	print *, "fv(1) after several polytope loops"
!	print "(f15.5)", fv(1)
!	print * ," "
!	call printvfv(n, v, fv)

end subroutine polytope

! ============================================================	
! ============================================================	
! END POLYTOPE EXECUTION
! (secondary subroutines below)
! ==========================================================
! ==========================================================


! =========================================================================================
! =========================================================================================

	subroutine ctr_simplex(n, xin, usual_delta, zero_term_delta, v, fv,  &
							NN, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda)

		implicit none
		integer(4), intent(in) :: n
		real(8), dimension(n), intent(in) :: xin
		real(8), intent(in) :: usual_delta, zero_term_delta
		real(8), dimension(n, n+1), intent(inout) :: v
		real(8), dimension(n+1), intent(inout) :: fv
		
		real(8), dimension(n) :: y
		integer(4) :: j
		
		! MODEL VARIABLES/DATA/PARAMETERS
		! ==================================================================================
		integer(4), intent(in) :: NN
		real(8), dimension(NN), intent(in) :: w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib
		integer(4), dimension(NN), intent(in) :: availability
		real(8), intent(in) :: thetaa, thetab
		real(8), dimension(NN), intent(inout) :: mylambda

		! ===========================================================
		! SIMPLEX
		do j = 1, n
			y = xin
			if (y(j) /= 0) then
				y(j) = (1 + usual_delta)*y(j)
			else
				y(j) = zero_term_delta 
			end if
			v(1:n, j+1) = y
			call calc_obj(NN, n, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda, y, fv(j+1))
		end do
		
	end subroutine	ctr_simplex

! =========================================================================================
! =========================================================================================

	subroutine reset_permute(np1, permute)
		integer(4), intent(in) :: np1
		integer(4), dimension(np1), intent(inout) :: permute	!permutation matrix	
		integer(4) :: j
		do j = 1, np1
			permute(j) = j
		end do
	end subroutine	reset_permute
	
	subroutine poly_loop(n, np1, two, prho, pchi, ppsi, psigma, tol_fun, &
						tol_par, maxfun, maxiter, func_evals, itercount, v, fv, how, success, &
						NN, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda)
		implicit none
	
		integer(4), intent(in) :: n			!number of variables to choose
		integer(4), intent(in) :: np1	
		integer(4), intent(in) :: two	
		real(8), intent(in) :: prho, pchi, ppsi, psigma, tol_fun, tol_par
		integer(4), intent(in) :: maxfun, maxiter
		integer(4), intent(inout) :: func_evals, itercount
		real(8), dimension(n, np1), intent(inout) :: v
		real(8), dimension(np1), intent(inout) :: fv

		character (len = 30), intent(out) :: how, success
		integer(4), dimension(np1) :: permute	!permutation matrix
		real(8), dimension(n) :: xbar, xe, xr, xc, xcc
		real(8) :: fxr, fxe, fxc, fxcc
		integer(4) :: j

		! MODEL VARIABLES/DATA/PARAMETERS
		integer(4), intent(in) :: NN
		real(8), dimension(NN), intent(in) :: w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib
		integer(4), dimension(NN), intent(in) :: availability
		real(8), intent(in) :: thetaa, thetab
		real(8), dimension(NN), intent(inout) :: mylambda

		! =================================================================
		while_loop: do

		! =============================================================
		! STOP COUNTROLS
		! =============================================================
		if ((maxval(abs(fv(2:np1) - fv(1:n)))<= tol_fun) .and. (maxval(abs(v(:, 2:np1) - v(:, 1:n)))<= tol_par)) then
			success = "minimum found"
			print *, success, "after", itercount, "iterations"
			print *, " "
			exit while_loop
		else if ((func_evals >= maxfun) .or. (itercount >= maxiter)) then
			success = "minimum not found"
			print *, success, "after", itercount, "iterations"
			print *, " "
			exit while_loop
		end if
		! ================= END STOP COUNTROLS ========================

		xbar = sum(v(:, 1:n), 2)/n
		xr = (1 + prho)*xbar - prho*v(:, np1)
		where (xr < 0) xr = 0.9*xbar
		
		call calc_obj(NN, n, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda, xr, fxr)
		func_evals = func_evals + 1
		
		if1: if (fxr <= fv(1)) then
			! CALCULATE EXPANSION POINT
			xe = (1 + prho*pchi)*xbar - prho*pchi*v(:, np1)
			where (xe < 0) xe = 0.9*xbar
			
			call calc_obj(NN, n,availability,  w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda, xe, fxe)
			func_evals = func_evals + 1
			if (fxe < fxr) then
				v(:, np1) = xe;
				fv(np1) = fxe;
				how = "expand"
			else
				v(:, np1) = xr;
				fv(np1) = fxr;
				how = "reflect"
			end if
		else	!(fv(1) <= fxr)
			if2: if (fxr < fv(n)) then
				v(:, np1) = xr
				fv(np1) = fxr
				how = "reflect"
			else !if2: fxr >= fv(n)
				
				! PERFORM CONTRACTION
				if31: if (fxr < fv(np1)) then
					! PERFORM OUTSIDE CONTRACTION
					xc = (1 + prho*ppsi)*xbar - prho*ppsi*v(:, np1)
					where (xc < 0) xc = 0.9*xbar
					
					call calc_obj(NN, n, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda, xc, fxc)
					func_evals = func_evals + 1
					if41: if (fxc <= fxr) then
						v(:, np1) = xc
						fv(np1) = fxc
						how = "contract outside"
					else !if41
						! PERFORM A SHRINK
						how = "shrink"
					end if if41
				else !if31
					! PERFORM INSIDE CONTRACTION
					xcc = (1 - ppsi)*xbar + ppsi*v(:, np1)
					where (xcc < 0) xcc = 0.9*xbar
					
					call calc_obj(NN, n, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda, xcc, fxcc)
					func_evals = func_evals + 1
					if42: if (fxcc < fv(np1)) then
						v(:, np1) = xcc
						fv(np1) = fxcc
						how = "contract inside"
					else !if42
						! PERFORM A SHRINK
						how = "shrink"
					end if if42
				end if if31
				if32: if (how == "shrink") then
					do j = 2, np1
						v(:, j) = v(:, 1) + psigma*(v(:, j) - v(:, 1))
						call calc_obj(NN, n, availability, w, w_highest, w_fourth, w_third, w_second, w_lowest, xsharea, myphia, myphib, thetaa, thetab, mylambda, v(:, j), fv(j))
					end do 
					func_evals = func_evals + 1
				end if if32
			end if if2							
		end if if1
		! ================ SORT V AND FV ===========================
		call reset_permute(np1, permute)
		call DSVRGP(np1, fv, fv, permute)				!sorts fv returning both sorted vector fv and permutation vector
		call DPERMA(n, np1, v, n, permute, two, v, n)	!permutates the columns of v according to the sorting of fv
		! ================ END SORT V AND FV ========================
		itercount = itercount + 1

		end do while_loop

	end subroutine poly_loop
