! ============================================================	
! READS DATA
! ==========================================================

subroutine readata(NN, H, w, wpop, XT99, distance, contig, lang, agree, uswage, w_highest, w_fourth, w_third, w_second, w_lowest)
				  
	implicit none
	
	integer(4), intent(in) :: NN
	real(8), dimension(NN), intent(out) :: H				! population	in millions
	real(8), dimension(NN), intent(out) :: w				! wages 1999 (normalized to US = 1)
	real(8), dimension(NN), intent(out) :: wpop				! working population (as a faction of total)
	real(8), dimension(NN, NN), intent(out) :: XT99			! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN), intent(out) :: distance		! distance in km
	integer(4), dimension(NN, NN), intent(out) :: contig		! 1 for contiguity
	integer(4), dimension(NN, NN), intent(out) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN), intent(out) :: agree			! 1 for both countries in the same trade agreement--ASEAN, COMESA, EFTA, MERCOSUR, NAFTA, EU
	real(8), intent(out) :: uswage
	real(8), dimension(NN), intent(out) :: w_highest, w_fourth, w_third, w_second, w_lowest

	real(8), dimension(NN) :: tdata		!temporary auxiliary dataset
	real(8), dimension(NN) :: share
	integer, dimension(NN) :: test

	integer(4) :: j, exp, imp
	
contig = 0
lang = 0
agree = 0

! ============================================================
! POPULATION IN MILLIONS

open(2, file = 'populationun_ineq.dat', STATUS ='OLD')
	do j=1,NN
		read(2,*), H(j)
	end do
close(2)
H = H/(1.e6)	!transform to millions

! ============================================================
! WAGES (NORMALIZED BY US WAGES)

open(2, file = 'percgdpun_ineq.dat', STATUS ='OLD')
	do j = 1, NN
		read(2,*), w(j)
	end do
close(2)
uswage = w(1)
w = w/uswage


! ============================================================
! WORKING POPULATION, AS A PERCENTAGE OF TOTAL

wpop = 1.

! ============================================================
! TRADE FLOWS (IN MILLIONS OF US$)

open(2, file = 'tradeflowun_ineq.dat', STATUS ='OLD')		!trade flows in US$000
	do exp = 1, NN
		do imp = 1, NN
			read(2,*), XT99(imp, exp)		!rows are importers
		end do
	end do
close(2)
XT99 = XT99/1e6
! set domestic flows to zero
do j = 1, NN		
	XT99(j,j) = 0
end do

! ============================================================
! DISTANCE BETWEEN COUNTRIES (MOST POPULATED CITY, IN 1000 KM)

open(2, file = 'distun_ineq.dat', STATUS ='OLD')
	do exp = 1, NN
		do imp = 1, NN
			read(2,*), distance(imp, exp)		!rows are importers
		end do
	end do
close(2)
distance = distance/1000	! transform to 000 km

! ============================================================
! CONTIGUITY (1 for contiguity, 0 otherwise)

open(2, file = 'contiguityun_ineq.dat', STATUS ='OLD')
	do exp = 1, NN
		do imp = 1, NN
			read(2,*), contig(imp, exp)		!rows are importers
		end do
	end do
close(2)


! ============================================================
! LANGUAGE (1 for common official language, 0 otherwise)

open(2, file = 'languageoun_ineq.dat', STATUS ='OLD')
	do exp = 1, NN
		do imp = 1, NN
			read(2,*), lang(imp, exp)		!rows are importers
		end do
	end do
close(2)

! ============================================================
! TRADE AGREEMENTS

open(2, file = 'agreementun_ineq.dat', STATUS ='OLD')
	do exp = 1, NN
		do imp = 1, NN
			read(2,*), agree(imp, exp)		!rows are importers
		end do
	end do
close(2)

! ============================================================
! WAGES FOR EACH INCOME GROUP (QUINTILES)

!wages in the highest quintile
open(2, file = 'qshare_highest_ineq.dat', STATUS ='OLD')
	do j=1,NN
		read(2,*), share(j)			! income share
	end do
close(2)
w_highest = (w*share)/.2

!wages in the fourth quintile
open(2, file = 'qshare_fourth_ineq.dat', STATUS ='OLD')
	do j=1,NN
		read(2,*), share(j)			! income share
	end do
close(2)
w_fourth = (w*share)/.2

!wages in the third quintile
open(2, file = 'qshare_third_ineq.dat', STATUS ='OLD')
	do j=1,NN
		read(2,*), share(j)			! income share
	end do
close(2)
w_third = (w*share)/.2

!wages in the second quintile
open(2, file = 'qshare_second_ineq.dat', STATUS ='OLD')
	do j=1,NN
		read(2,*), share(j)			! income share
	end do
close(2)
w_second = (w*share)/.2

!wages in the lowest quintile
w_lowest = (w - (w_highest*.2 + w_fourth*.2 + w_third*.2 + w_second*.2))/.2

! tests if wages are in the right order
test = 0
where (w_highest <= w_fourth) test = 1
where (w_fourth <= w_third) test = 1
where (w_third <= w_second) test = 1
where (w_second <= w_lowest) test = 1
where (w_lowest <= 0) test = 1
print *, "test must be equal to zero"
print *, "test = ", sum(test)
print *, " ------------------------------------------ "
print *, " "

! activate the lines below to eliminate income inequalities within countries
! ===========================================================================
! w_highest = w
! w_fourth = w
! w_third = w
! w_second = w
! w_lowest = w

end subroutine readata
