! ============================================================	
! READ DATA
! ==========================================================

subroutine readata(NN, H, w, wpop, XT99, distance, contig, lang, agree, uswage)
				  
	implicit none
	
	integer(4), intent(in) :: NN
	real(8), dimension(NN), intent(out) :: H				! population	in millions
	real(8), dimension(NN), intent(out) :: w				! wages 1999 (normalized to US = 1)
	real(8), dimension(NN), intent(out) :: wpop				! working population (as a faction of total)
	real(8), dimension(NN, NN), intent(out) :: XT99			! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN), intent(out) :: distance		! distance in km
	integer(4), dimension(NN, NN), intent(out) :: contig		! 1 for contiguity
	integer(4), dimension(NN, NN), intent(out) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN), intent(out) :: agree			! 1 for both countries in the same trade agreement--ASEAN, COMESA, EFTA, MERCOSUR, NAFTA, EU
	real(8), intent(out) :: uswage

	real(8), dimension(NN) :: tdata		!temporary auxiliary dataset
	integer(4) :: j, exp, imp
	
contig = 0
lang = 0
agree = 0

! ============================================================
! POPULATION IN MILLIONS

open(2, file = 'population.dat', STATUS ='OLD')
	do j=1,NN
		read(2,*), H(j)
	end do
close(2)
H = H/(1.e6)	!transform to millions

! ============================================================
! WAGES (NORMALIZED BY US WAGES)

open(2, file = 'percgdp.dat', STATUS ='OLD')
	do j = 1, NN
		read(2,*), w(j)
	end do
close(2)
uswage = w(1)
w = w/uswage


! ============================================================
! WORKING POPULATION, AS A PERCENTAGE OF TOTAL

wpop = 1.

! ============================================================
! TRADE FLOWS (IN MILLIONS OF US$)

open(2, file = 'tradeflow.dat', STATUS ='OLD')		!trade flows in US$000
	do exp = 1, NN
		do imp = 1, NN
			read(2,*), XT99(imp, exp)		!rows are importers
		end do
	end do
close(2)
XT99 = XT99/1e6
! set domestic flows to zero
do j = 1, NN		
	XT99(j,j) = 0
end do

! ============================================================
! DISTANCE BETWEEN COUNTRIES (MOST POPULATED CITY, IN 1000 KM)

open(2, file = 'dist.dat', STATUS ='OLD')
	do exp = 1, NN
		do imp = 1, NN
			read(2,*), distance(imp, exp)		!rows are importers
		end do
	end do
close(2)
distance = distance/1000	! transform to 000 km

! ============================================================
! CONTIGUITY (1 for contiguity, 0 otherwise)

open(2, file = 'contiguity.dat', STATUS ='OLD')
	do exp = 1, NN
		do imp = 1, NN
			read(2,*), contig(imp, exp)		!rows are importers
		end do
	end do
close(2)


! ============================================================
! LANGUAGE (1 for common official language, 0 otherwise)

open(2, file = 'languageo.dat', STATUS ='OLD')
	do exp = 1, NN
		do imp = 1, NN
			read(2,*), lang(imp, exp)		!rows are importers
		end do
	end do
close(2)

! ============================================================
! TRADE AGREEMENTS

open(2, file = 'agreement.dat', STATUS ='OLD')
	do exp = 1, NN
		do imp = 1, NN
			read(2,*), agree(imp, exp)		!rows are importers
		end do
	end do
close(2)


end subroutine readata
